// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.logdemo;

import org.objectweb.corba.runtime.*;

/**
 **  The application deployment.
 **/
public class LogDemo
implements Runnable
{
    // main
    private ORBService _orb_service;

    public
    LogDemo(ORBService orbs)
    {
        // main
        _orb_service = orbs;
    }

    //
    //  java.lang.Runnable
    //

    final public void
    run()
    { 
        try {
            // obtain the INS scheme of the registration service
            RegistrationService regs = org.objectweb.corba.runtime.Runtime.getRegistrationService();
            INSRegistrationScheme scheme = (INSRegistrationScheme)regs.get_scheme(INSRegistrationScheme.SCHEME_ID);

            // obtain homes
            System.err.println("## Obtain homes");
            ClientHome chome = ClientHomeHelper.narrow(scheme.lookup("ClientHome", _orb_service));
            ServerHome shome = ServerHomeHelper.narrow(scheme.lookup("ServerHome", _orb_service));
            AsyncServerHome ashome = AsyncServerHomeHelper.narrow(scheme.lookup("AsyncServerHome", _orb_service));

            // create components
            System.err.println("## Create components");
            Client client1 = chome.create();
            Client client2 = chome.create();
            Server server = shome.create();
            AsyncServer aserver = ashome.create();

            // configure
            System.err.println("## Configure components");
            client1.name("Mathieu");
            client2.name("Vincent");
            server.name("Pascal");
            aserver.name("Async Pascal");
            // connect display facet/receptacle
            System.err.println("## Connect client/server");
            SynchronousDisplay dis = server.provide_display();
            client1.connect_display(dis);
            client2.connect_display(dis);
            // connect message consumer/emitter
            System.err.println("## Connect client/asyncserver");
            AsyncMessageConsumer cons = aserver.get_consumer_async_display();
            client1.connect_async_display(cons);
            client2.connect_async_display(cons);

            // complete configuration
            System.err.println("## Complete configuration");
            client1.configuration_complete();
            client2.configuration_complete();
            server.configuration_complete();
            aserver.configuration_complete();
        } catch (Exception ex) {
            throw new Error(ex.getMessage());
        }

        System.out.println("Again ....");
        System.exit(1);
    }

    static public Runnable
    main(ORBService orbs)
    {
        return new LogDemo(orbs);
    }
}







