// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Default DOM parser for the CORBA Component Service Descriptor (CCSD) XML DTD.</p>
 **/
public class CCSDParser
extends org.xml.sax.helpers.DefaultHandler
{
    // ccsd 
    static final private String _class_name = "CCSDParser";
    private ORBService _orbservice;

    // state machine
    private boolean _serviceid_found;
    private boolean _reqcorbaservice_found;
    private boolean _corbaserviceref_found;

    // built fields
    private String _service_id;
    private org.omg.CORBA.Object _csref_ref;
    private String _csref_type;
    private String _csref_regname;

    // default constructor
    public
    CCSDParser(ORBService orbs)
    {
        // ccsd 
        _orbservice = orbs;

        // state machine
        _serviceid_found = false;
        _reqcorbaservice_found = false;
        _corbaserviceref_found = false;

        // built fields
        _service_id = null;
        _csref_ref = null;
        _csref_type = null;
        _csref_regname = null;
    }

    //
    //  internal operations
    // 

    static private void
    parseAsSAXDoc(String fname, String dtdloc,
                  org.xml.sax.helpers.DefaultHandler handler)
    {
        final String opname = "parseAsSAXDoc";
        // create SAX input source
        org.xml.sax.InputSource isource = null;
        try {
            isource = new org.xml.sax.InputSource(new java.io.FileInputStream(fname));
        } catch (java.io.FileNotFoundException ex) {
            final String msg = "FAILED (file not found: "+fname+")";
            TheLogger.error(_class_name, opname, msg, ex);
            return ;
        }

        // change location of DTD in document
        String loc = isource.getSystemId();
        isource.setSystemId(dtdloc+"/"+loc);

        // obtain new SAX parser factory
        // NOTE: disable namespaces and validation
        javax.xml.parsers.SAXParserFactory fact = javax.xml.parsers.SAXParserFactory.newInstance();
        fact.setNamespaceAware(false);
        fact.setValidating(false);

        // create SAX parser and parse XML file
        try {
            javax.xml.parsers.SAXParser parser = fact.newSAXParser();
            parser.parse(isource, handler);
        } catch (Exception ex) {
            TheLogger.error(_class_name, opname, "FAILED", ex);
            return ;
        }
    }    

    //
    //  static public operations
    //

    static public CCSDParser
    parse(String fname, String dtdloc, ORBService orbs)
    {
        CCSDParser parser = new CCSDParser(orbs);
        parseAsSAXDoc(fname, dtdloc, parser);

        return parser;
    }

    //
    //  public operations
    //

    final public String
    getServiceId()
    {
        return _service_id;
    }

    final public org.coach.ECM.CORBAServiceRef
    getCORBAServiceRef()
    {
        final String opname = "getCORBAServiceRef";
        if (_csref_ref==null) {
            final String msg = "no CORBAServiceRef element defined";
            TheLogger.debug(_class_name, opname, msg);
            return null;
        }

        org.coach.ECM.CORBAServiceRef csref = new org.coach.ECM.CORBAServiceRef();
        if (_csref_type==null) {
            final String msg = "type attribute of requirescorbaservice element must be specified";
            TheLogger.error(_class_name, opname, msg);
        }

        if (_csref_regname==null) {
            final String msg = "registerwithorb element must be specified";
            TheLogger.error(_class_name, opname, msg);
        }

        csref.corba_type = _csref_type;
        csref.orb_registration_name = _csref_regname;
        csref.initial_reference = _csref_ref;

        return csref;
    }

    //
    //  org.xml.sax.ContentHandler
    //

    final public void
    startDocument()
    throws org.xml.sax.SAXException
    {
        //final String opname = "startDocument";
        //TheLogger.debug(_class_name, opname, "starting document");
    }

    final public void
    endDocument()
    throws org.xml.sax.SAXException
    {
        //final String opname = "endDocument";
        //TheLogger.debug(_class_name, opname, "ending document");
    }

    final public void
    startElement(String nsuri, String lname, String qname,
                 org.xml.sax.Attributes attrs)
    throws org.xml.sax.SAXException
    {
        // equiv XPath req: "corbacomponentservice/serviceid"
        if (qname.equals("serviceid")) {
            _serviceid_found = true;
            return ;
        }

        // equiv XPath req: "corbacomponentservice/requirescorbaservice"
        // equiv XPath req: "corbacomponentservice/requirescorbaservice/@type"
        if (qname.equals("requirescorbaservice")) {
            _reqcorbaservice_found = true;
            _csref_type = attrs.getValue("type");
            return ;
        }

        // equiv XPath req: "corbacomponentservice/requirescorbaservice/corbaserviceref"
        if (_reqcorbaservice_found && (qname.equals("corbaserviceref"))) {
            _corbaserviceref_found = true;
            return ;
        }

        // equiv XPath req: "corbacomponentservice/requirescorbaservice/registerwithorb/@name"
        if (_reqcorbaservice_found && (qname.equals("registerwithorb"))) {
            _csref_regname = attrs.getValue("name");
            return ;
        }

        // equiv XPath req: "corbacomponentservice/requirescorbaservice/corbaserviceref/objref/@string"
        if (_corbaserviceref_found && (qname.equals("objref"))) {
            String ior = attrs.getValue("string");
            _csref_ref = _orbservice.string_to_object(ior);
            return ;
        }
        // equiv XPath req: "corbacomponentservice/requirescorbaservice/corbaserviceref/link/@href"
        if (_corbaserviceref_found && (qname.equals("link"))) {
            // TODO
            return ;
        }
        // equiv XPath req: "corbacomponentservice/requirescorbaservice/corbaserviceref/ins/@name"
        if (_corbaserviceref_found && (qname.equals("ins"))) {
            String insname = attrs.getValue("name");
            RegistrationService regs = org.objectweb.corba.runtime.Runtime.getRegistrationService();
            INSRegistrationScheme scheme = (INSRegistrationScheme)regs.get_scheme(INSRegistrationScheme.SCHEME_ID);

            _csref_ref = scheme.lookup(insname, _orbservice);
            return ;
        }

    }

    final public void
    endElement(String nsuri, String lname, String qname)
    throws org.xml.sax.SAXException
    {
        // equiv XPath req: "corbacomponentservice/serviceid"
        if (qname.equals("serviceid")) {
            _serviceid_found = false;
            return ;
        }

        // equiv XPath req: "corbacomponentservice/requirescorbaservice"
        if (qname.equals("requirescorbaservice")) {
            _reqcorbaservice_found = false;
            return ;
        }

        // equiv XPath req: "corbacomponentservice/requirescorbaservice/corbaserviceref"
        if (_reqcorbaservice_found && (qname.equals("corbaserviceref"))) {
            _corbaserviceref_found = false;
            return ;
        }
    }

    final public void
    characters(char[] ch, int start, int length)
    throws org.xml.sax.SAXException
    {
        // equiv XPath req: "corbacomponentservice/serviceid/text()"
        if (_serviceid_found) {
            _service_id = new String(ch, start, length);
            //final String opname = "characters";
            //final String msg = "code entrypoint found: "+code_entrypoint;
            //TheLogger.debug(_class_name, opname, msg);
            return ;
        }
    }
}
