// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p></p>
 **/
public class ServiceArchiveImpl
extends ArchiveServantBase
implements Servant, org.coach.ECM.ServiceArchiveOperations
{
    // system component
    static final private String _class_name = "ServiceArchiveImpl";

    // servant
    static final private String _type_id =  org.coach.ECM.ServiceArchiveHelper.id();

    // archive
    private ORBService _orb_service;
    private String _uuid;
    private String _archive_location;
    private String _dtd_location;

    // service archive
    private String _service_id;
    private String _code_location;
    private String _code_entrypoint;
    private String _csd_location;
    private String _ccsd_location;
    // fully scope as CORBAServiceRef is also a java class in this package
    private org.coach.ECM.CORBAServiceRef _corbaservice_ref;

    // default constructor
    public
    ServiceArchiveImpl(ORBService orbs,
                       String dtdloc,
                       String uuid,
                       String loc)
    {
        super(uuid, loc);

        // archive
        _orb_service = orbs;
        _uuid = uuid;
        _archive_location = loc;
        _dtd_location = dtdloc;

        // service archive
        _service_id = null;
        _code_location = null;
        _code_entrypoint = null;
        _csd_location = null;
        _ccsd_location = null;
        _corbaservice_ref = null;

        // initiliaze some internal fields
        initializeInternalFields();
    }

    //
    //  abstract operations implementation
    //

    final protected ORBService
    getORBService()
    {
        return _orb_service;
    }

    //
    //  internal operations
    //

    private void 
    initializeInternalFields()
    {
        final String opname = "initializeInternalFields";
        // open zip
        java.util.zip.ZipFile zip = openZip();

        // extract CSD location
        java.util.Enumeration entries = zip.entries();
        java.util.zip.ZipEntry entry = null;
        java.util.zip.ZipEntry csdentry  = null;
        while (entries.hasMoreElements()) {
            entry = (java.util.zip.ZipEntry)entries.nextElement();
            if (entry.getName().endsWith(".csd")) {
                csdentry = entry;
            }
        }

        // check if CSD found
        if (csdentry==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (no CSD found in "+_uuid+")";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        // copy CSD file locally
        String csdfname = extractFile(zip, csdentry);
        _csd_location = csdfname;

        // parse CSD file
        CSDParser csdparser = CSDParser.parse(_csd_location, _dtd_location, _uuid);

        // obtain code entry point
        _code_entrypoint = csdparser.code_entrypoint;

        // extract CCSD and code archive
        entries = zip.entries();
        java.util.zip.ZipEntry ccsdentry  = null;
        java.util.zip.ZipEntry codelocentry  = null;
        String ccsdname = csdparser.descriptor_location;
        String codeloc = csdparser.code_location;
        while (entries.hasMoreElements()) {
            entry = (java.util.zip.ZipEntry)entries.nextElement();
            if (entry.getName().equals(ccsdname)) {
                ccsdentry = entry;
            }
            else if (entry.getName().equals(codeloc)) {
                codelocentry = entry;
            }
        }

        // check if code archive found
        if (codelocentry==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (no code library found in "+_uuid+")";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        // copy code archive locally
        String codelocfname = extractFile(zip, codelocentry);
        _code_location = codelocfname;

        // check if CCSD found
        if (ccsdentry==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (CCSD file not found in "+_uuid+")";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        // copy CCSD file locally
        String ccsdfname = extractFile(zip, ccsdentry);
        _ccsd_location = ccsdfname;

        // parse CCSD 
        CCSDParser ccsdparser = CCSDParser.parse(_ccsd_location, _dtd_location, getORBService());

        // get service id
        _service_id = ccsdparser.getServiceId();

        // get corbaservice ref
        _corbaservice_ref = ccsdparser.getCORBAServiceRef();

        // close CSD
        // NOTE: how ?
        // close CCSD
        // NOTE: how ?

        // close zip
        try {
            zip.close();
        } catch(java.io.IOException ex) {
            // NOTE: TODO: should be reported as an exception
            // ignore for the moment
            TheLogger.error(_class_name, opname, "IGNORE", ex);
        }
    }

    //
    //  IDL:org/objectweb/corba/runtime/SystemComponent:1.0
    //

    final public void
    system_configuration_complete(SystemConfiguration cfg)
    {
        // initiliaze some internal fields
        initializeInternalFields();
    }

    final public void
    destroy()
    {
        // TODO: destroy local files extracted from the zip
    }

    //
    //  IDL:org/objectweb/corba/runtime/Servant:1.0
    //

    final public String
    type_id()
    {
        return _type_id;
    }

    final public org.omg.PortableServer.Servant
    as_native_servant()
    {
        return new org.coach.ECM.ServiceArchivePOATie(this);
    }


    //
    //  IDL:org/objectweb/ccm/runtime/Archive:1.0
    //

    // public String get_fileinarchive_location(String name, String link)
    // public String get_href_location(String href)
    // public String get_link_location(String link)
    // public String get_ins_as_objref(String ins)
    // public String uuid()

    //
    //  IDL:coach.org/ECM/ServiceArchive:1.0
    //

    final public String
    service_id()
    {
        return _service_id;
    }

    final public String 
    csd_location()
    {
        return _csd_location;
    }

    final public String
    ccsd_location()
    {
        return _ccsd_location;
    }

    final public String
    code_location()
    {
        return _code_location;
    }

    final public String
    code_entrypoint()
    {
        return _code_entrypoint;
    }

    final public org.coach.ECM.CORBAServiceRef
    get_corbaservice_ref()
    throws org.coach.ECM.NoCORBAServiceRefRequired
    {
        if (_corbaservice_ref==null) {
            throw new org.coach.ECM.NoCORBAServiceRefRequired();
        }

        return _corbaservice_ref;
    }

    final public String
    exec_xpath_request_in_csd(String req)
    {
        final String opname = "exec_xpath_request_in_csd";
        final String msg = "FAILED (not implemented, rethrown)";
        TheLogger.debug(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public String
    exec_xpath_request_in_ccsd(String req)
    {
        final String opname = "exec_xpath_request_in_ccsd";
        final String msg = "FAILED (not implemented, rethrown)";
        TheLogger.debug(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }
}
