// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p></p>
 **/
public class HomeLocalEventsImpl
{
    // event
    static private String _class_name = "HomeLocalEventsImpl";
    private java.util.Hashtable _id2channel;
    private java.util.Hashtable _id2inter;
    private java.util.Hashtable _name2id;
    private int _id_count;
    private CallInterceptorFactory _factory;
 
    // empty constructor
    public
    HomeLocalEventsImpl()
    {
        // event
        _id2channel = null;
        _id2inter = null;
        _name2id = null;
        _id_count = 0;
        _factory = null;
    }

    // default constructor
    public
    HomeLocalEventsImpl(CallInterceptorFactory ifact)
    {
        // event
        _id2channel = new java.util.Hashtable();
        _id2inter = new java.util.Hashtable();
        _name2id = new java.util.Hashtable();
        _id_count = 0;
        _factory = ifact;
    }

    //
    //  IDL:omg.org/Components/LocalEvents:1.0
    //

    final public int
    get_channel(byte[] cid, String name)
    throws org.omg.Components.ChannelUnavailable
    {
        // check if a channel is already created
        String rname = new String(cid)+":"+name;
        Integer chid = (Integer)_name2id.get(rname);
        if (chid!=null) {
            return chid.intValue();
        }

        // create id
        int nchid = _id_count++;

        // create instance
        HomeFakeEventChannel channel = new HomeFakeEventChannel(nchid);

        // create interceptor if available
        if (_factory!=null) {
            // NOTE: used an untyped interceptor as the API of the context disable the use
            // of typed operations (therefore a typed interceptor is useless)
            org.omg.CORBA.Object inter = _factory.createUntypedStubInterceptor(name, channel);
            _id2inter.put(new Integer(nchid), inter);
        }

        // store
        _name2id.put(rname, new Integer(nchid));
        _id2channel.put(new Integer(nchid), channel);

        return nchid;
    }

    final public org.omg.Components.Cookie
    subscribe(byte[] cid, org.omg.Components.EventConsumerBase consumer, int chid)
    throws org.omg.Components.InvalidChannel
    {
        // get channel
        HomeFakeEventChannel channel = (HomeFakeEventChannel)_id2channel.get(new Integer(chid));

        if (channel==null) {
            throw new org.omg.Components.InvalidChannel();
        }

        return channel.subscribe(consumer);
    }

    final public org.omg.Components.EventConsumerBase
    unsubscribe(byte[] cid, org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidSubscription
    {
        // NOTE: ck must be a CookieImpl
        if (!(ck instanceof CookieImpl)) {
            // NOTE: is this the right exception ? there's no other one ...
            throw new org.omg.Components.InvalidSubscription();
        }

        // get channel id from cookie
        CookieImpl lck = (CookieImpl)ck;
        int chid = lck.getChannelId();

        // get channel
        HomeFakeEventChannel channel = (HomeFakeEventChannel)_id2channel.get(new Integer(chid));

        if (channel==null) {
            throw new org.omg.Components.InvalidSubscription();
        }

        return channel.unsubscribe(ck);
    }

    final public void
    push(byte[] cid, org.omg.Components.EventBase evt, int chid)
    throws org.omg.Components.InvalidChannel
    {
        org.omg.Components.EventConsumerBase cons = null;
        if (_factory!=null) {
            // use interceptor as EventConsumerBase
             cons = (org.omg.Components.EventConsumerBase)_id2inter.get(new Integer(chid));
        } else {
            // use directly the channel
            cons = (org.omg.Components.EventConsumerBase)_id2channel.get(new Integer(chid));
        }

        if (cons==null) {
            throw new org.omg.Components.InvalidChannel();
        }

        try {
            cons.push_event(evt);
        } catch (org.omg.Components.BadEventType ex) {
            // ignore
        }
    }
}


//
class HomeFakeEventChannel
extends org.omg.Components._EventConsumerBaseStub
{
    // event channel
    private String _class_name = "ECMFakeEventChannel";
    private int _channel_id;
    private java.util.Hashtable _ck2consumers;

    // default constructor
    public
    HomeFakeEventChannel(int chid)
    {
        // event channel
        _channel_id = chid;
        _ck2consumers = new java.util.Hashtable();
    }

    //
    //  public operations
    //

    final public org.omg.Components.Cookie
    subscribe(org.omg.Components.EventConsumerBase consumer)
    {
        // create local cookie
        org.omg.Components.Cookie lck = new CookieImpl(_channel_id);

        // store
        _ck2consumers.put(lck, consumer);

        return lck;
    }

    final public org.omg.Components.EventConsumerBase
    unsubscribe(org.omg.Components.Cookie ck)
    throws org.omg.Components.InvalidSubscription
    {
        // remove from storage
        org.omg.Components.EventConsumerBase cons = (org.omg.Components.EventConsumerBase)_ck2consumers.remove(ck);

        // check cookie
        if (cons==null) {
            throw new org.omg.Components.InvalidSubscription();
        }

        //
        return cons;
    }

    //
    //  IDL:omg.org/Components/EventConsumerBase:1.0
    //

    final public void
    push_event(org.omg.Components.EventBase evt)
    throws org.omg.Components.BadEventType
    {
        org.omg.Components.EventConsumerBase[] consumers = null;
        consumers = (org.omg.Components.EventConsumerBase[])_ck2consumers.values().toArray(new org.omg.Components.EventConsumerBase[0]);

        //String opname = "push_event";
        //String msg = "pushing events to "+consumers.length+" consumers";
        //TheLogger.debug(_class_name, opname, msg);

        // NOTE: use a worker for each consumer
        for (int i=0;i<consumers.length;i++) {
            new HomePushEventWorker(consumers[i], evt).start();
        }
    }
}

// NOTE: should use a thread pool
class HomePushEventWorker
extends java.lang.Thread
{
    //
    private String _class_name = "ECMPushEventWorker";
    private org.omg.Components.EventConsumerBase _consumer;
    private org.omg.Components.EventBase _event;

    // default constructor
    public
    HomePushEventWorker(org.omg.Components.EventConsumerBase cons,
                        org.omg.Components.EventBase evt)
    {
        //
        _consumer = cons;
        _event = evt;
    }

    //
    //  java.lang.Thread
    //

    final public void
    run()
    {
        try {
            _consumer.push_event(_event);
        } catch (org.omg.Components.BadEventType ex) {
            // ignore
            String opname = "run";
            TheLogger.debug(_class_name, opname, "IGNORE (bad event type)");
       }
    }
}