@echo off
REM ==================================================================== 
REM
REM   ECM: The Extensible Container Model
REM   Copyright (C) 2004 THALES
REM   Contact: openccm-ecm@objectweb.org
REM
REM   This library is free software; you can redistribute it and/or
REM   modify it under the terms of the GNU Lesser General Public
REM   License as published by the Free Software Foundation; either
REM   version 2.1 of the License, or any later version.
REM
REM   This library is distributed in the hope that it will be useful,
REM   but WITHOUT ANY WARRANTY; without even the implied warranty of
REM   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
REM   Lesser General Public License for more details.
REM
REM   You should have received a copy of the GNU Lesser General Public
REM   License along with this library; if not, write to the Free Software
REM   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
REM   USA
REM
REM   Initial developer(s): Mathieu Vadet
REM   Initial funding: IST COACH European Project (IST-2001-34445)
REM                    http://www.ist-coach.org
REM
REM ==================================================================== 


REM
REM  Starts a component server
REM

setlocal

goto begin

:usage
    echo usage: cs_start.bat [options]
    echo Starts a CCM ComponentServer.
    echo --
    echo The following options are supported:
    echo   /h, /help    Print this message
    echo   /debug       Activate debug mode (optional)
    echo   /ns nsior    NS to use for registrations, default to NS_at_hostname NS (optional)
    echo   name         The logical name of the CS, no default value (mandatory)
    exit /B 1

:begin
set debug=
set name=
set ns=

:parse_args
if not "%1"=="" (
    if "%1"=="/h" (
        goto usage
    ) else if "%1"=="/help" (
        goto usage
    ) else if "%1"=="/debug" (
        set debug=/debug
    ) else if "%1"=="/ns" (
        set ns=/ns %2
        shift
    ) else (
        set name=%1
    )

    shift
    goto parse_args
)

REM check if name is set
if not defined name (
    echo ERROR: logical name of the CCM ComponentServer is missing.
    exit /B 1
)

REM obtain ExtCI IOR and set it as an initial ref
call @ECM.install.bin.dir@\extci_ior.bat %debug% /url > %TEMP%\extci_ior.tmp
REM check if the IOR was correctly retrieved
if ERRORLEVEL 1 exit /B 1

for /f %%i in (%TEMP%\extci_ior.tmp) do (
    set args=-ORBInitRef ExtComponentInstallation=%%i %args%
)
del %TEMP%\extci_ior.tmp 2> nul

REM set code location and entrypt
set location=@ECM.install.lib.dir@\ccm_runtime_cs.jar
set entrypt=org.objectweb.ccm.runtime.ComponentServerImpl.create_servant

REM delegate to corba_runtime_start script
call @ECM.install.bin.dir@\corba_runtime_start.bat /args "%args:"=%" /location %location% /entrypoint %entrypt% %ns% %debug% %name%

:eof
