#! /bin/sh
# ==================================================================== 
#
#   ECM: The Extensible Container Model
#   Copyright (C) 2004 THALES
#   Contact: openccm-ecm@objectweb.org
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
#   Initial developer(s): Mathieu Vadet
#   Initial funding: IST COACH European Project (IST-2001-34445)
#                    http://www.ist-coach.org
#
# ==================================================================== 


#
#  Starts a CORBA runtime with a specific root object
#

usage() {
    echo "usage: corba_runtime_start [options]"
    echo "Starts a CORBA runtime."
    echo "--"
    echo "The following options are supported:"
    echo "  -h, --help                      Print this message"
    echo "  -d, --debug                     Activate debug mode (optional)"
    echo "  --location   location           Set JAR location (optional)"
    echo "  --entrypoint entrypoint         Set servant entrypoint (mandatory)"
    echo "  --ns nsior                      NS to use for registrations, default to NS_at_hostname NS (optional)"
    echo "  --jvmargs    \"arg1 ... argn\"    Set arg1 ... argn as JVM arguments (optional)"
    echo "  --args       \"arg1 ... argn\"    Set arg1 ... argn as launcher arguments (optional) "
    echo "  name                            The logical name of the CORBA runtime, no default value (mandatory)"
    exit /B 1
}

# parse arguments
debug=
location=
entrypoint=
nsior=
args=
jvmargs=
name=

while [ -n "$1" ] ;
do
    if [ "$1" = "-h" ] || [ "$1" = "--help" ];
    then
        usage ;
    elif [ "$1" = "-d" ] || [ "$1" = "--debug" ] ;
    then
        debug=--debug ;
    elif [ "$1" = "--location" ] ;
    then
        location=$2
        shift ;
    elif [ "$1" = "--entrypoint" ] ;
    then
        entrypoint=$2
        shift ;
    elif [ "$1" = "--ns" ] ;
    then
        nsior=$2
        shift ;
    elif [ "$1" = "--args" ] ;
    then
        args=$2
        shift ;
    elif [ "$1" = "--jvmargs" ] ;
    then
        jvmargs=$2
        shift ;
    else
        name=$1 ;
    fi

    shift ;
done


# check if name is set
if [ -z "$name" ] ;
then
    echo "ERROR: logical name of the CORBA runtime is missing."
    exit 1;
fi

# check if entrypoint is set
if [ -z  "$entrypoint" ] ;
then
    echo "ERROR: entrypoint of the servant is missing."
    exit 1;
fi

# Check if the CORBA Runtime is started.
if [ -f $CORBA_RUNTIME_REPOSITORY/$name.pid ] ;
then
    if [ -n "$debug" ] ;
    then
        echo "WARNING: The CORBA runtime with logical name: $name is already started." ;
    fi ;
fi

# check if a NS is provided
if [ -z "$nsior" ] ;
then
    # use a local NS
    nsior=`@ECM.install.bin.dir@/ns_ior.sh $debug --url` ;
fi

# starts the CORBA Runtime
if [ -n "$debug" ] ;
then
    echo "INFO: The CORBA Runtime $name will be started ..." ;
fi

# add debug if needed
if [ -n "$debug" ] ;
then
    jvmargs="-Ddebug=true $jvmargs"
    args="-DebugOn" ;
else
    jvmargs="-Ddebug=false $jvmargs"
    args="-DebugOff" ;
fi

# add archive location if needed
if [ -n "$location" ] ;
then
    jvmargs="-Darchive.location=$location $jvmargs"
    args="-ArchiveLocation $location $args" ;
fi

# add entrypoint and runtime id
jvmargs="-Darchive.entrypoint=$entrypoint $jvmargs"
jvmargs="-Druntime.id=$name $jvmargs"
args="-ArchiveEntrypoint $entrypoint $args"
args="-RuntimeId $name $args"

# add IOR file
jvmargs="-Druntime.iorfile=$CORBA_RUNTIME_REPOSITORY/$name.ior $jvmargs"

args="--runid corba_runtime -ORBInitRef NameService=$nsior $args"
@OpenCCM.install.bin.dir@/launcher.sh fork "$jvmargs" "$args @ECM.install.launcher.dir@/corba_runtime.xml" $CORBA_RUNTIME_REPOSITORY/$name.output $CORBA_RUNTIME_REPOSITORY/$name.pid

if [ -n "$debug" ] ;
then
    echo "INFO: The CORBA Runtime $name is started."
fi
