// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;
// import of OpenCCM modules
import org.objectweb.corba.ast.api.*;
import org.objectweb.corba.ast.lib.*;
import org.objectweb.corba.generator.java.lib.*;
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;
import org.objectweb.corba.generator.common.lib.GenerationException;

/**
 **  <p>Generator class for client-side and server-side interceptors.</p>
 **/
public class CIDLCOPIGenerator
extends org.objectweb.corba.generator.common.lib.GeneratorBase
{
    // generator
    private ExtendedJavaTranslator _translator;
    private RepositoryImpl _repository; 

    // default constructor
    public
    CIDLCOPIGenerator(AST ast)
    {
        super(ast);

        _translator = new ExtendedJavaTranslator();
        _repository = null;
    }

    //
    //  internal operations
    //

    private ExtendedJavaTranslator
    getTranslator()
    {
        return _translator;
    }

    private RepositoryImpl
    getRepository()
    {
        return _repository;
    }

    // NOTE: not used currently, but will be in the future (maybe)
    private void
    generateForModule(ModuleDecl module)
    {
        Declaration[] decls = null;
        // generate for all enclosed components
        decls = module.getContents(false, DeclarationKind.dk_component);
        for (int i=0;i<decls.length;i++) {
            generateForComponent((ComponentDecl)decls[i]);
        }

        // generate for all enclosed homes
        decls = module.getContents(false, DeclarationKind.dk_home);
        for (int i=0;i<decls.length;i++) {
            generateForHome((HomeDecl)decls[i]);
        }

        // recursively generate for all enclosed modules
        decls = module.getContents(false, DeclarationKind.dk_module);
        for (int i=0;i<decls.length;i++) {
            generateForModule((ModuleDecl)decls[i]);
        }
    }

    private void
    generateForHome(HomeDecl home)
    {
        // generate skeleton interceptor for equiv itf
        // generateSkI(home.getClientMapping(), home);

        // interceptor factory
        generateIFactory(home);
    }

    private void
    generateForComponent(ComponentDecl comp)
    {
        // NOTE: skeleton interceptors are generated by segments, not by facet/sink
        // As CIDL is currently not supported, we assume a monolithic component
        // (i.e. all facets/sinks in the main segment)
        Declaration[] facets = comp.getContents(false, DeclarationKind.dk_provides);
        Declaration[] sinks =  comp.getContents(false, DeclarationKind.dk_consumes);
        Declaration[] mainseg = new Declaration[facets.length+sinks.length+1];

        mainseg[0] = comp;
        System.arraycopy(facets, 0, mainseg, 1, facets.length);
        System.arraycopy(sinks, 0, mainseg, facets.length+1, sinks.length);
        generateForSegment(org.omg.Components.COMPONENT_ID.value, comp, mainseg);

        // receptacle
        Declaration[] decls = null;
        decls = comp.getContents(false, DeclarationKind.dk_uses);
        for (int i=0;i<decls.length;i++) {
            // generate stub interceptor for used itf
            UsesDecl rec = (UsesDecl)decls[i];
            InterfaceDecl itf = rec.getInterface();
            // check if the used interface is a home interface
            // in that case, use the equivalent interface
            if (itf.getDeclKind()==DeclarationKind.dk_home) {
                HomeDecl home = (HomeDecl)itf;
                itf = home.getClientMapping();
            }

            generateStubI(itf, rec);
        }

        // emitter
        decls = comp.getContents(false, DeclarationKind.dk_emits);
        for (int i=0;i<decls.length;i++) {
            // generate stub interceptor for consumer itf
            EmitsDecl emitter = (EmitsDecl)decls[i];
            generateStubI(emitter.getEvent().getClientMapping(), emitter);
        }

        // publisher
        decls = comp.getContents(false, DeclarationKind.dk_publishes);
        for (int i=0;i<decls.length;i++) {
            // generate stub interceptor for consumer itf
            PublishesDecl publisher = (PublishesDecl)decls[i];
            generateStubI(publisher.getEvent().getClientMapping(), publisher);
        }
    }

    private void
    generateIFactory(HomeDecl home)
    {
        ComponentDecl comp = home.getManagedComponent();
        String classname = home.getName()+"_IFact";
        ClassObject ski = new ClassObjectImpl(classname);

        // header
        ski.setPackage(getTranslator().getPackage(comp));
        ski.addImport("org.objectweb.corba.runtime.*");
        ski.addImport("org.objectweb.ccm.runtime.*");

        // class decl
        // * public class $classname
        //      implements CallInterceptorFactory
        ski.addComment(" ");
        ski.addComment("Java interceptor factory for component "+comp.getName()); 
        ski.addComment(" ");
        ski.addComment("NOTE: generated class; DO NOT EDIT !");
        ski.addComment(" ");
        ski.setModifier(ModifierKindImpl.mk_public);
        ski.addImplementedObject("CallInterceptorFactory");

        // internal state
        AttributeObject attr = null;
        // * static private _class_name = "<class_name>";
        attr = new AttributeObjectImpl();
        attr.addComment("Class name of this class (used for debugging).");
        attr.setStatic(true);
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("String");
        attr.setName("_class_name");
        attr.setInitialValue("\""+classname+"\"");
        ski.addAttribute(attr);

        // * private ORBService _orb_service;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ORBService");
        attr.setName("_orb_service");
        ski.addAttribute(attr);

        // * private ServicesSet _services_set;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ServicesSet");
        attr.setName("_services_set");
        ski.addAttribute(attr);

        // * private String _uuid;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("String");
        attr.setName("_uuid");
        ski.addAttribute(attr);

        // default constructor
        // * protected $classname()
        ConstructorObject constructor = new ConstructorObjectImpl();
        constructor.getImpl().setMacro("IFACT_CONSTRUCTOR_BODY");
        ski.addConstructor(constructor);

        // entrypoint
        // * static public CallInterceptorFactory create_factory()
        MethodObject operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setStatic(true);
        operation.setModifier(ModifierKindImpl.mk_public);
        operation.setReturnType("CallInterceptorFactory");
        operation.setName("create_factory");
        operation.getImpl().setMacro("IFACT_CREATEFACTORY_BODY");
        operation.getImpl().addContextValue("classname", classname);
        ski.addMethod(operation);

        // public operation
        // * final public void setContextInfo(ORBService orbs, ServicesSet sset, String uuid)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setFinal(true);
        operation.setModifier(ModifierKindImpl.mk_public);
        operation.setReturnType("void");
        operation.setName("setContextInfo");
        ParameterObject parameter = new ParameterObjectImpl();
        parameter.setName("orbs");
        parameter.setType("ORBService");
        operation.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("sset");
        parameter.setType("ServicesSet");
        operation.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("uuid");
        parameter.setType("String");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("IFACT_SETCONTEXTINFO_BODY");
        ski.addMethod(operation);

        // * public org.omg.PortableServer.Servant
        //   createSkeletonInterceptor(short sid, org.omg.CORBA.Object executor)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setFinal(true);
        operation.setModifier(ModifierKindImpl.mk_public);
        operation.setReturnType("org.omg.PortableServer.Servant");
        operation.setName("createSkeletonInterceptor");
        parameter = new ParameterObjectImpl();
        parameter.setName("sid");
        parameter.setType("short");
        operation.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("executor");
        parameter.setType("org.omg.CORBA.Object");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("IFACT_CREATESKINTERCEPTOR_BODY");
        // NOTE: only manages monolithic at present (always return main seg interceptor)
        String mainseg_classname = comp.getName()+"_MainSegSkI";
        operation.getImpl().addContextValue("mainseg_classname", mainseg_classname);
        ski.addMethod(operation);

        // * public org.omg.CORBA.Object
        //   createStubInterceptor(String name, org.omg.CORBA.Object delegate)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setFinal(true);
        operation.setModifier(ModifierKindImpl.mk_public);
        operation.setReturnType("org.omg.CORBA.Object");
        operation.setName("createStubInterceptor");
        parameter = new ParameterObjectImpl();
        parameter.setName("name");
        parameter.setType("String");
        operation.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("delegate");
        parameter.setType("org.omg.CORBA.Object");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("IFACT_CREATESTUBINTERCEPTOR_BODY");
        Declaration[] receptacles = comp.getContents(false, DeclarationKind.dk_uses);
        Declaration[] emitters = comp.getContents(false, DeclarationKind.dk_emits);
        Declaration[] publishers = comp.getContents(false, DeclarationKind.dk_publishes);
        Boolean has_receptacles = new Boolean((receptacles.length!=0));
        Boolean has_emitters = new Boolean((emitters.length!=0));
        Boolean has_publishers = new Boolean((publishers.length!=0));
        operation.getImpl().addContextValue("receptacles", receptacles);
        operation.getImpl().addContextValue("emitters", emitters);
        operation.getImpl().addContextValue("publishers", publishers);
        operation.getImpl().addContextValue("has_receptacles", has_receptacles);
        operation.getImpl().addContextValue("has_emitters", has_emitters);
        operation.getImpl().addContextValue("has_publishers", has_publishers);
        ski.addMethod(operation);

        // * public org.omg.CORBA.Object
        //   createUntypedStubInterceptor(String name, org.omg.CORBA.Object delegate)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setFinal(true);
        operation.setModifier(ModifierKindImpl.mk_public);
        operation.setReturnType("org.omg.CORBA.Object");
        operation.setName("createUntypedStubInterceptor");
        parameter = new ParameterObjectImpl();
        parameter.setName("name");
        parameter.setType("String");
        operation.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("delegate");
        parameter.setType("org.omg.CORBA.Object");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("IFACT_CREATEUNTYPEDSTUBINTERCEPTOR_BODY");
        operation.getImpl().addContextValue("emitters", emitters);
        operation.getImpl().addContextValue("publishers", publishers);
        operation.getImpl().addContextValue("has_emitters", has_emitters);
        operation.getImpl().addContextValue("has_publishers", has_publishers);
        ski.addMethod(operation);

        // add to repository
        getRepository().addObject(ski);
    }

    private void
    generateForSegment(short segid, ComponentDecl comp, Declaration[] idl3decls)
    {
        String classname = "";
        String ssegid = java.lang.Short.toString(segid);
        if (segid==org.omg.Components.COMPONENT_ID.value) {
            // main segment
            classname = comp.getName()+"_MainSegSkI";
        } else {
            // other segment
            classname = comp.getName()+"_Seg"+ssegid+"SkI";
        }

        ClassObject ski = new ClassObjectImpl(classname);

        // header
        ski.setPackage(getTranslator().getPackage(comp));
        ski.addImport("org.objectweb.corba.runtime.*");
        ski.addImport("org.objectweb.ccm.runtime.*");

        // class decl
        // * public class $classname
        //      extends $comp.getName()_MainSegPOA
        // or 
        // * public class $classname
        //      extends $comp.getName()_Seg${segid}POA
        ski.addComment(" ");
        if (segid==org.omg.Components.COMPONENT_ID.value) {
            ski.addComment("Java server-side interceptor for main segment of component "+comp.getName()); 
        } else {
            ski.addComment("Java server-side interceptor for segment "+ssegid+" of component "+comp.getName()); 
        }
        ski.addComment(" ");
        ski.addComment("NOTE: generated class; DO NOT EDIT !");
        ski.addComment(" ");
        ski.setModifier(ModifierKindImpl.mk_public);
        if (segid==org.omg.Components.COMPONENT_ID.value) {
            ski.addInheritedObject(comp.getName()+"_MainSegPOA");
        } else {
            ski.addInheritedObject(comp.getName()+"_Seg"+ssegid+"POA");
        }

        // internal state
        AttributeObject attr = null;
        // * static private _class_name = "<class_name>";
        attr = new AttributeObjectImpl();
        attr.addComment("Class name of this class (used for debugging).");
        attr.setStatic(true);
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("String");
        attr.setName("_class_name");
        attr.setInitialValue("\""+classname+"\"");
        ski.addAttribute(attr);

        // * private ORBService _orb_service;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ORBService");
        attr.setName("_orb_service");
        ski.addAttribute(attr);

        // * private ServicesSet _services_set;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ServicesSet");
        attr.setName("_services_set");
        ski.addAttribute(attr);

        // * private String _uuid;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("String");
        attr.setName("_uuid");
        ski.addAttribute(attr);

        // * private ServerCallContextImpl _call_ctx;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ServerCallContextImpl");
        attr.setName("_call_ctx");
        ski.addAttribute(attr);

        // * private java.util.Hashtable _informations;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("java.util.Hashtable");
        attr.setName("_informations");
        ski.addAttribute(attr);

        // * private $comp.getName()_MainSegOperations _delegate;
        // or
        // * private $comp.getName()_Seg${segid}Operations _delegate;
        attr = new AttributeObjectImpl();
        attr.addComment("Typed delegate.");
        attr.setModifier(ModifierKindImpl.mk_private);
        String delegate_type = null;
        if (segid==org.omg.Components.COMPONENT_ID.value) {
            delegate_type = comp.getName()+"_MainSegOperations";
        } else {
            delegate_type = comp.getName()+"_Seg"+ssegid+"Operations";
        }
        attr.setType(delegate_type);
        attr.setName("_delegate");
        ski.addAttribute(attr);

        // default constructor
        // * protected $classname(ORBService orbs, ServicesSet sset, String uuid,
        //                        org.omg.CORBA.Object executor)
        ConstructorObject constructor = new ConstructorObjectImpl();
        // params
        ParameterObject parameter = new ParameterObjectImpl();
        parameter.setName("orbs");
        parameter.setType("ORBService");
        constructor.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("sset");
        parameter.setType("ServicesSet");
        constructor.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("uuid");
        parameter.setType("String");
        constructor.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("executor");
        parameter.setType("org.omg.CORBA.Object");
        constructor.addParameter(parameter);
        // initialize all fields
        constructor.getImpl().addContextValue("delegate_type", delegate_type);
        constructor.getImpl().setMacro("SKI_CONSTRUCTOR_BODY");
        ski.addConstructor(constructor);

        // internal operation
        // * final protected CallInformation _getCallInformation(String sname)
        MethodObject operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setModifier(ModifierKindImpl.mk_private);
        operation.setReturnType("CallInformation");
        operation.setName("_getCallInformation");
        parameter = new ParameterObjectImpl();
        parameter.setName("name");
        parameter.setType("String");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("SKI_GETCALLINFORMATION_BODY");
        ski.addMethod(operation);

        // * private short _preinvoke(org.coach.ECA.ServerCallController[] ctrls)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setModifier(ModifierKindImpl.mk_private);
        operation.setReturnType("short");
        operation.setName("_preinvoke");
        parameter = new ParameterObjectImpl();
        parameter.setName("ctrls");
        parameter.setType("org.coach.ECA.ServerCallController[]");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("SKI_PREINVOKE_BODY");
        ski.addMethod(operation);

        // * private short _postinvoke(short fstate)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setModifier(ModifierKindImpl.mk_private);
        operation.setReturnType("short");
        operation.setName("_postinvoke");
        parameter = new ParameterObjectImpl();
        parameter.setName("fstate");
        parameter.setType("short");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("SKI_POSTINVOKE_BODY");
        ski.addMethod(operation);

        // generation for all declaration of the segment
        for (int i=0;i<idl3decls.length;i++) {
            generateSkIOps(ski, idl3decls[i]);
        }

        // add to repository
        getRepository().addObject(ski);
    }

    private void
    generateSkIOps(ClassObject ski, Declaration idl3decl)
    {
        InterfaceDecl itf = null;
        String sname = "::"+idl3decl.getName();

        if (idl3decl.getDeclKind()==DeclarationKind.dk_component) {
            ComponentDecl comp = (ComponentDecl)idl3decl;
            itf = comp.getClientMapping();
        } else if (idl3decl.getDeclKind()==DeclarationKind.dk_provides) {
            ProvidesDecl facet = (ProvidesDecl)idl3decl;
            itf = facet.getInterface();

            // its a facet, include component name
            sname = "::"+idl3decl.getParent().getName()+sname;
        } else if (idl3decl.getDeclKind()==DeclarationKind.dk_consumes) {
            ConsumesDecl sink = (ConsumesDecl)idl3decl;
            itf = sink.getEvent().getClientMapping();

            // its a sink, include component name
            sname = "::"+idl3decl.getParent().getName()+sname;
        }

        // wrapped business operations
        Declaration[] decls = itf.getContents(false, DeclarationKind.dk_operation);
        OperationDecl op = null;
        for (int i=0;i<decls.length;i++) {
            op = (OperationDecl)decls[i];

            String ciname = null;
            if (idl3decl.getDeclKind()==DeclarationKind.dk_consumes) {
                // NOTE: do not include op name in case of an operation coming
                // from an event port
                ciname = sname;
            } else {
                ciname = sname+"::"+op.getName();
            }

            addWrappedOperation(ski, op, op.getType(), ciname, 
                                op.getParameterList().getParameters(),
                                op.getExceptionList().getExceptions(),
                                "SKI_WRAPPED_OPERATION");
        }

        // wrapped attributes
        decls = itf.getContents(false, DeclarationKind.dk_attribute);
        AttributeDecl attrdecl = null;
        for (int i=0;i<decls.length;i++) {
            attrdecl = (AttributeDecl)decls[i];

            // check readonly 
            if (attrdecl.isReadonly()) {
                // getter (no parameter)
                addWrappedOperation(ski, attrdecl, attrdecl.getType(), sname+"::_get_"+attrdecl.getName(), 
                                    new Parameter[0], attrdecl.getExceptions(),
                                    "SKI_WRAPPED_OPERATION");
            }
            else {
                // getter (no parameter)
                addWrappedOperation(ski, attrdecl, attrdecl.getType(), sname+"::_get_"+attrdecl.getName(), 
                                    new Parameter[0], attrdecl.getGetExceptions(),
                                    "SKI_WRAPPED_OPERATION");

                // setter (parameter is attribute type)
                // use attribute name as parameter name
                Parameter[] params = new Parameter[] {
                    new ParameterImpl(attrdecl.getName(), attrdecl.getType(), org.omg.CORBA.ParameterMode.PARAM_IN)
                };

                addWrappedOperation(ski, attrdecl, getAST().getVoidType(), sname+"::_set_"+attrdecl.getName(), 
                                    params, attrdecl.getSetExceptions(),
                                    "SKI_WRAPPED_OPERATION");

            }
        }
    }

    private void
    generateStubI(InterfaceDecl itf, Declaration idl3decl)
    {
        String classname = "";

        // check declaration kind to build the class name
        if (idl3decl.getDeclKind()==DeclarationKind.dk_uses) {
            classname = idl3decl.getParent().getName()+"_"+idl3decl.getName()+"StubI";
        }
        if (idl3decl.getDeclKind()==DeclarationKind.dk_emits) {
            classname = idl3decl.getParent().getName()+"_"+idl3decl.getName()+"StubI";
        }
        if (idl3decl.getDeclKind()==DeclarationKind.dk_publishes) {
            classname = idl3decl.getParent().getName()+"_"+idl3decl.getName()+"StubI";
        }

        ClassObject ski = new ClassObjectImpl(classname);

        // header
        ski.setPackage(getTranslator().getPackage(itf));
        ski.addImport("org.objectweb.corba.runtime.*");
        ski.addImport("org.objectweb.ccm.runtime.*");

        // class decl
        // * public class $classname
        //      extends _$itf.getName()Stub
        ski.addComment(" ");
        ski.addComment("Java client-side interceptor for interface "+itf.getName()); 
        ski.addComment(" ");
        ski.addComment("NOTE: generated class; DO NOT EDIT !");
        ski.addComment(" ");
        ski.setModifier(ModifierKindImpl.mk_public);
        ski.addInheritedObject("_"+itf.getName()+"Stub");

        // internal state
        AttributeObject attr = null;
        // * static private _class_name = "<class_name>";
        attr = new AttributeObjectImpl();
        attr.addComment("Class name of this class (used for debugging).");
        attr.setStatic(true);
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("String");
        attr.setName("_class_name");
        attr.setInitialValue("\""+classname+"\"");
        ski.addAttribute(attr);

        // * private ORBService _orb_service;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ORBService");
        attr.setName("_orb_service");
        ski.addAttribute(attr);

        // * private ServicesSet _services_set;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ServicesSet");
        attr.setName("_services_set");
        ski.addAttribute(attr);

        // * private String _uuid;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("String");
        attr.setName("_uuid");
        ski.addAttribute(attr);

        // * private ClientCallContextImpl _call_ctx;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("ClientCallContextImpl");
        attr.setName("_call_ctx");
        ski.addAttribute(attr);

        // * private java.util.Hashtable _informations;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("java.util.Hashtable");
        attr.setName("_informations");
        ski.addAttribute(attr);

        // * private org.omg.Components.EventConsumerBase _consbase;
        attr = new AttributeObjectImpl();
        attr.addComment(".");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType("org.omg.Components.EventConsumerBase");
        attr.setName("_consbase");
        ski.addAttribute(attr);

        // * private $itf.getName() _delegate;
        attr = new AttributeObjectImpl();
        attr.addComment("Typed delegate.");
        attr.setModifier(ModifierKindImpl.mk_private);
        attr.setType(getTranslator().getAbsoluteName(itf));
        attr.setName("_delegate");
        ski.addAttribute(attr);

        // default constructor
        // * protected $classname(ORBService orbs, ServicesSet sset, String uuid, org.omg.CORBA.Object del)
        ConstructorObject constructor = new ConstructorObjectImpl();
        // params
        ParameterObject parameter = new ParameterObjectImpl();
        parameter.setName("orbs");
        parameter.setType("ORBService");
        constructor.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("sset");
        parameter.setType("ServicesSet");
        constructor.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("uuid");
        parameter.setType("String");
        constructor.addParameter(parameter);
        parameter = new ParameterObjectImpl();
        parameter.setName("del");
        parameter.setType("org.omg.CORBA.Object");
        constructor.addParameter(parameter);
        // initialize all fields
        constructor.getImpl().setMacro("STUBI_CONSTRUCTOR_BODY");
        constructor.getImpl().addContextValue("interface", itf);
        ski.addConstructor(constructor);

        // internal operation
        // * final protected CallInformation _getCallInformation(String sname)
        MethodObject operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setModifier(ModifierKindImpl.mk_private);
        operation.setReturnType("CallInformation");
        operation.setName("_getCallInformation");
        parameter = new ParameterObjectImpl();
        parameter.setName("name");
        parameter.setType("String");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("STUBI_GETCALLINFORMATION_BODY");
        ski.addMethod(operation);

        // * private short _preinvoke(org.coach.ECA.ClientCallController[] ctrls)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setModifier(ModifierKindImpl.mk_private);
        operation.setReturnType("short");
        operation.setName("_preinvoke");
        parameter = new ParameterObjectImpl();
        parameter.setName("ctrls");
        parameter.setType("org.coach.ECA.ClientCallController[]");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("STUBI_PREINVOKE_BODY");
        ski.addMethod(operation);

        // * private short _postinvoke(short fstate)
        operation = new MethodObjectImpl();
        operation.addComment(".");
        operation.setModifier(ModifierKindImpl.mk_private);
        operation.setReturnType("short");
        operation.setName("_postinvoke");
        parameter = new ParameterObjectImpl();
        parameter.setName("fstate");
        parameter.setType("short");
        operation.addParameter(parameter);
        operation.getImpl().setMacro("STUBI_POSTINVOKE_BODY");
        ski.addMethod(operation);

        // build scoped name
        // NOTE: idl3decl may be a receptacle / emitter / publisher
        String sname = "::"+idl3decl.getName();
        if (idl3decl.getDeclKind()==DeclarationKind.dk_uses) {
            // its a receptacle, include component name
            sname = "::"+idl3decl.getParent().getName()+sname;
        }
        if (idl3decl.getDeclKind()==DeclarationKind.dk_emits) {
            // its an emitter, include component name
            sname = "::"+idl3decl.getParent().getName()+sname;
        }
        if (idl3decl.getDeclKind()==DeclarationKind.dk_publishes) {
            // its a pubisher, include component name
            sname = "::"+idl3decl.getParent().getName()+sname;
        }

        // wrapped business operations
        Declaration[] decls = itf.getContents(false, DeclarationKind.dk_operation);
        OperationDecl op = null;
        for (int i=0;i<decls.length;i++) {
            op = (OperationDecl)decls[i];


            addWrappedOperation(ski, op, op.getType(), sname+"::"+op.getName(), 
                                op.getParameterList().getParameters(),
                                op.getExceptionList().getExceptions(),
                                "STUBI_WRAPPED_OPERATION");
        }

        // wrapped attributes
        decls = itf.getContents(false, DeclarationKind.dk_attribute);
        AttributeDecl attrdecl = null;
        for (int i=0;i<decls.length;i++) {
            attrdecl = (AttributeDecl)decls[i];

            // check readonly 
            if (attrdecl.isReadonly()) {
                // getter (no parameter)
                addWrappedOperation(ski, attrdecl, attrdecl.getType(), sname+"::_get_"+attrdecl.getName(), 
                                    new Parameter[0], attrdecl.getExceptions(),
                                    "STUBI_WRAPPED_OPERATION");
            }
            else {
                // getter (no parameter)
                addWrappedOperation(ski, attrdecl, attrdecl.getType(), sname+"::_get_"+attrdecl.getName(), 
                                    new Parameter[0], attrdecl.getGetExceptions(),
                                    "STUBI_WRAPPED_OPERATION");

                // setter (parameter is attribute type)
                // use attribute name as parameter name
                Parameter[] params = new Parameter[] {
                    new ParameterImpl(attrdecl.getName(), attrdecl.getType(), org.omg.CORBA.ParameterMode.PARAM_IN)
                };

                addWrappedOperation(ski, attrdecl, getAST().getVoidType(), sname+"::_set_"+attrdecl.getName(), 
                                    params, attrdecl.getSetExceptions(),
                                    "STUBI_WRAPPED_OPERATION");

            }
        }

        // add to repository
        getRepository().addObject(ski);
    }

    private void
    addWrappedOperation(ClassObject ski, Declaration decl, TypeRef rtype, String ciname,
                        Parameter[] params, ExceptionDecl[] exceptions,
                        String macroname)
    {
        addWrappedOperation(ski, decl, rtype, ciname, params, exceptions, macroname, new String[0], new Object[0]);
    }

    private void
    addWrappedOperation(ClassObject ski, Declaration decl, TypeRef rtype, String ciname,
                        Parameter[] params, ExceptionDecl[] exceptions,
                        String macroname, String ctxkey, Object ctxval)
    {
        String[] ctxkeys = { ctxkey };
        Object[] ctxvals = { ctxval };

        addWrappedOperation(ski, decl, rtype, ciname, params, exceptions, macroname, ctxkeys, ctxvals);
    }

    private void
    addWrappedOperation(ClassObject ski, Declaration decl, TypeRef rtype, String ciname,
                        Parameter[] params, ExceptionDecl[] exceptions,
                        String macroname, String[] ctxkeys, Object[] ctxvals)
    {
        MethodObject operation = new MethodObjectImpl();
        // return type (java mapping)
        operation.setReturnType(getTranslator().toJava(rtype));
        
        // name
        operation.setName(decl.getName());
        
        // parameters (java mapping)
        ParameterObject parameter = null;
        for (int j=0;j<params.length;j++) {
            parameter = new ParameterObjectImpl();
            parameter.setName(params[j].getName());
            parameter.setType(getTranslator().toJava(params[j].getType(), params[j].getMode()));
            operation.addParameter(parameter);
        }
        // build parameters list
        String param_names = "";
        // check if params set is not empty
        if (params.length!=0) {
            for (int j=0;j<params.length-1;j++) {
                param_names = param_names + params[j].getName() + ", ";
            }
            
            param_names = param_names + params[params.length-1].getName();
        }
        operation.getImpl().addContextValue("parameter_list", param_names);
        
        // exceptions
        if (exceptions.length!=0) {
            operation.getImpl().addContextValue("has_exceptions", new Boolean(true));
            String[] exc_absnames = new String[exceptions.length];
            for (int j=0;j<exceptions.length;j++) {
                exc_absnames[j] = getTranslator().getAbsoluteName(exceptions[j]);
                    operation.addException(exc_absnames[j]);
            }
            operation.getImpl().addContextValue("exception_absnames", exc_absnames);
        }
        else {
                operation.getImpl().addContextValue("has_exceptions", new Boolean(false));
        }
        
        // body
        // NOTE: only the body's macro differs from a client-side to a server-side wrapped operation
        operation.getImpl().setMacro(macroname);
        operation.getImpl().addContextValue("declaration", decl);
        operation.getImpl().addContextValue("callinfo_name", ciname);
        operation.getImpl().addContextValue("return_type", getTranslator().toJava(rtype));
        
        // add external context values
        for (int i=0;i<ctxkeys.length;i++) {
            operation.getImpl().addContextValue(ctxkeys[i], ctxvals[i]);
        }

        if (rtype.getTypeKind()!=TypeKind.tk_void) {
            operation.getImpl().addContextValue("has_return_type", new Boolean(true));
            // any extractor and insertor
            String return_type_extractor = getTranslator().toJavaAnyExtractor(rtype);
            String return_type_insertor = getTranslator().toJavaAnyInsertor(rtype);
            // return null value
            String return_null_value = getTranslator().toJavaNullValue(rtype);
            if (rtype.isDeclaration()) {
                return_type_extractor = return_type_extractor+"(_call_ctx.result())";
                return_type_insertor = return_type_insertor+"(anyres, res)";
            } 
            else {
                return_type_extractor = "_call_ctx.result()."+return_type_extractor+"()";
                return_type_insertor = "anyres."+return_type_insertor+"(res)";
            }
            operation.getImpl().addContextValue("return_type_extractor", return_type_extractor);
            operation.getImpl().addContextValue("return_type_insertor", return_type_insertor);
            operation.getImpl().addContextValue("return_null_value", return_null_value);
        }
        else {
            operation.getImpl().addContextValue("has_return_type", new Boolean(false));
        }
        
        
        // add operation to class
        ski.addMethod(operation);
    }

    //
    //  public operations
    //

    final public void
    generate(String basedir, Scope scope)
    throws GenerationException
    {
        _repository = new RepositoryImpl();

        // obtain objects for generation
        // check scope, should be:
        //   - a component
        //   - a module
        if (scope.getDeclKind()==DeclarationKind.dk_component) {
            generateForComponent((ComponentDecl)scope);
        }
        else if (scope.getDeclKind()==DeclarationKind.dk_module) {
            generateForModule((ModuleDecl)scope);
        }
        else {
            System.err.println("object is not a component: aborting");
            return ;
        }

        // NOTE: issue: the template file "ski.vm" is not located in the OpenCCM template dir,
        // but in the directory pointed by the "ecm.templates" property
        // It could be usefull to be able to specify externally the template dirs
        // in the initialization of the Java generator (along with the template files
        // themselves)
        // Currently, we are forced to copy the template file in the OpenCCM dir ...

        String tpath = System.getProperty("ecm.templates");
        java.io.File tdir = new java.io.File(tpath);

        // Generate Java Files
        JavaGenerator javagen = new JavaGenerator();
        java.util.Vector v = new java.util.Vector();
        v.add("ski.vm");
        v.add("stubi.vm");
        v.add("ifact.vm");
        javagen.initialize("Container Interceptors", v);
        javagen.generate(getRepository(), basedir);
    }

}


