// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

/**
 **  <p>.</p>
 **/
public class CIDLIDL
extends org.objectweb.corba.command.lib.CommandOnASTBase
{
    //
    private org.objectweb.util.cmdline.lib.DefaultOptionArgument _outdir;

    // default constructor
    public
    CIDLIDL()
    {
        super(new org.objectweb.util.cmdline.lib.DefaultCommandLine(
                   "cidl_idl",
                   "declaration",
                   "Generate the IDL mapping for a component/module from the OpenCCM IR3",
                   true));

        // Option: -d <outputdir>
        _outdir = new org.objectweb.util.cmdline.lib.DefaultOptionArgument(
                        new String[] { "-d", "--outputdir" }, "outputdir",
                        new String[] { "Generate IDL file into directory <outputdir>" },
                        null);

        // mandatory option
        _outdir.setMandatory(true);

        getCommandLine().addOption(_outdir);
    }

    //
    //  org.objectweb.corba.command.api.Application
    //

    final public int
    run(java.lang.String[] args)
    {
        // init the OpenCCM AST
        if(!initAST()) {
            return -1;
        }

        // obtain scope
        String declname = args[0];
        org.objectweb.corba.ast.api.Declaration decl = getAST().lookup(declname);

        if(decl==null) {
            getConsole().error("Declaration "+declname +" not found");
            return -1;
        }

        //
        getConsole().message("Generating implementations for " + declname + "...");
        CIDLIDLGenerator gen = new CIDLIDLGenerator(getAST());

        try {
            gen.initialize(getConsole());
            gen.generate(_outdir.getArgument(), decl, getConsole());
        } catch(org.objectweb.corba.generator.common.lib.GenerationException ex) {
            report_exception(ex);
            return -1;
        }

        return 0;
    }

    //
    //  bootstrap
    //

    static public void
    main(String[] args)
    {
        CIDLIDL cmd = new CIDLIDL();
        cmd.runMain(args);
    }
}
