@echo off
REM ==================================================================== 
REM
REM   ECM: The Extensible Container Model
REM   Copyright (C) 2004 THALES
REM   Contact: openccm-ecm@objectweb.org
REM
REM   This library is free software; you can redistribute it and/or
REM   modify it under the terms of the GNU Lesser General Public
REM   License as published by the Free Software Foundation; either
REM   version 2.1 of the License, or any later version.
REM
REM   This library is distributed in the hope that it will be useful,
REM   but WITHOUT ANY WARRANTY; without even the implied warranty of
REM   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
REM   Lesser General Public License for more details.
REM
REM   You should have received a copy of the GNU Lesser General Public
REM   License along with this library; if not, write to the Free Software
REM   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
REM   USA
REM
REM   Initial developer(s): Mathieu Vadet
REM   Initial funding: IST COACH European Project (IST-2001-34445)
REM                    http://www.ist-coach.org
REM
REM ==================================================================== 


REM WARNING: this file requires to activate the delayed variable expansion
REM do cmd /V=ON to activate it
REM test if delayed expansion is activated
for %%p in (a b) do (
    set TEST=!TEST!%%p
)

if "%TEST%"=="!TEST!b" (
    echo WARNING: this file requires to activate the delayed variable expansion
    echo do cmd /V=ON to activate it and restart
    goto eof
)

REM clean some variables
set ORB_HOME=
set ORB_NAME=
set INSTALL_DIR=
set CLASSPATH=
set default_install_dir=

REM source variables
call make_config.bat

REM JDK (no choice)
set JDK_HOME=%sun_jdk_1_4_0_dir%
set JDK_NAME=%sun_jdk_1_4_0_name%
set JDK_BIN=%JDK_HOME%\bin

REM Ant (no choice)
set ANT_HOME=%ant_1_6_0%

REM OpenCCM (no choice)
set OPENCCM_HOME=%openccm_0_8_0%

goto begin

REM usage
:usage
    echo make.bat [options]
    echo Builds or installs the ECM.
    echo --
    echo The following options are supported:
    echo   /h, /help    Print this message
    echo   /build       Build the ECM with a specific ORB
    echo   /demos       Build the ECM demonstrations with a specific ORB
    echo   /install     Install a built ECM in the filesystem
    exit /B 1

REM obtain build configuration
:select_orb
    echo Select ORB: 1) OpenORB 1.3.1    2) OpenORB 1.4.0    3) ORBacus 4.1.0
    set /P choice="Enter choice: "
    if "%choice%"=="1" (
        set ORB_HOME=%openorb_1_3_1_dir%
        set ORB_NAME=%openorb_1_3_1_name%
        echo Will use !ORB_NAME!
    ) else if "%choice%"=="2" (
        set ORB_HOME=%openorb_1_4_0_dir%
        set ORB_NAME=%openorb_1_4_0_name%
        echo Will use !ORB_NAME!
    ) else if "%choice%"=="3" (
        set ORB_HOME=%orbacus_4_1_0_dir%
        set ORB_NAME=%orbacus_4_1_0_name%
        echo Will use !ORB_NAME!
    ) else (
        echo Invalid choice: %choice%
        goto select_orb
    )

    goto !jumpback!


REM obtain installation configuration
:install_orb
    if defined default_install_dir (
        set jumpback=install_orb_jb
        goto select_orb

        :install_orb_jb
        set INSTALL_DIR=%default_install_dir%\!ORB_NAME!
        goto install_orb_end
    ) else (
       set /P INSTALL_DIR="Installation directory: "
    )

    :install_orb_end
    set MAKE_FILE=main\ant\install.xml
    set props=!props! -DECM.install.dir=%INSTALL_DIR%
    echo Will install ECM in: %INSTALL_DIR%

    goto jump1


REM parse arguments
:begin
set params=
set props=
set MAKE_FILE=main/ant/ecm.xml
set opt=

:parse_args
if not "%1"=="" (
    if "%1"=="/h" (
        goto usage
    ) else if "%1"=="/help" (
        goto usage
    ) else if "%1"=="/install" ( 
        set opt=t
        goto install_orb
    ) else if "%1"=="/build" (
        set opt=t
        set jumpback=jump1
        goto select_orb
    ) else if "%1"=="/demos" (
        set opt=t
        set MAKE_FILE=main\ant\ecm.xml demos
        set jumpback=jump1
        goto select_orb
    ) else if "%1"=="/clean" (
        set opt=t
        set MAKE_FILE=main\ant\ecm.xml clean
    ) else if "%1"=="/tasks" (
        set opt=t
        set MAKE_FILE=main\ant\tasks.xml
    ) else if "%1"=="/release" (
        set opt=t
        set MAKE_FILE=main\ant\ecm.xml release
    ) else if "%1"=="/doc" (
        set opt=t
        set MAKE_FILE=main\ant\ecm.xml doc
    ) else (
        REM NOTE: re-build the parameter list without our /xx options
        REM as ant will not understand it ...
        set params=!params! %1
    )

    shift
    goto parse_args
)


:jump1
REM check that an option was set
if not defined opt (
    echo ERROR: "/build", "/demos" or "/install" option expected.
    exit /B 1
)

REM add JDK bin to PATH
set PATH=%JDK_BIN%;%PATH%


REM set classpath for ant
REM Concatenate all the libraries in the %ANT_HOME% directory and
REM append them to the classpath environment variable.
for %%j in (%ANT_HOME%\lib\*.jar) do (
    set CLASSPATH=%%j;!CLASSPATH!
)

set CLASSPATH=%JDK_HOME%\lib\tools.jar;%CLASSPATH%

REM add ECM tasks to path
set CLASSPATH=%CD%\externals\Ant-1.6.0\lib\ecm_ant_tasks.jar;%CLASSPATH%
REM add ant-contrib to path
set CLASSPATH=%CD%\externals\Ant-1.6.0\lib\ant-contrib-0.6.jar;%CLASSPATH%

REM source OpenCCM if not already done
if not defined OpenCCM_HOMEDIR (
    call %OPENCCM_HOME%\bin\envi_OpenCCM.bat
)

REM delegate to Ant
%JDK_BIN%\java -cp %CLASSPATH% -Dant.home=%ANT_HOME% -Dant.dir=%ANT_HOME% -Djava.name=%JDK_NAME% -Djava.dir=%JDK_HOME% -DECM.dir=%CD% -DOpenCCM.install.dir=%OPENCCM_HOME% -DORB.name=%ORB_NAME% -DORB.dir=%ORB_HOME% %props% org.apache.tools.ant.Main %params% -q -buildfile %MAKE_FILE%

:eof
