// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Default implementation of the <tt>ComponentPOAService</tt> OMG IDL interface.</p>
 **/
public class ComponentPOAServiceImpl
extends org.omg.CORBA.LocalObject
implements ComponentPOAService
{
    // component poa service
    static final private String _class_name = "ComponentPOAServiceImpl";
    static final private String _service_id = ComponentPOAService.SERVICE_ID;

    private ORBService _orb_service;
    private ServicesSet _services_set;
    private org.omg.PortableServer.POA _home_poa;
    private org.omg.PortableServer.POA _component_poa;
    private ComponentServantActivatorImpl _servant_activator;

    // default constructor
    public
    ComponentPOAServiceImpl()
    {
        // component poa service
        _orb_service = null;
        _services_set = null;
        _home_poa = null;
        _component_poa = null;
        _servant_activator = null;
    }

    //
    //  internal operation
    //

    static public CallInterceptorFactory
    repid2Factory(String repid)
    {
        // NOTE: obtain the interceptor factory entrypoint from the home repository id
        final String opname = "repid2Factory";

        // Repid format: IDL:m1/.../mn/home:1.0 (without prefix)
        //               IDL:prefix/m1/.../mn/home:1.0 (with prefix)

        int idx1 = repid.indexOf(':');
        int idx2 = repid.lastIndexOf(':');
        // body       = home  or  m1/.../mn/home       or  prefix/m1/.../mn/home (prefix=p1.p2.pn)
        // java class = home  or  m1.m2.mn.home_IFact  or  pn.p2.p1.m1.m2.mn.home_IFact
        String body = repid.substring(idx1+1, idx2);
        String entrypt = null;

        idx1 = body.indexOf('.');
        if (idx1==-1) {
            // replace '/' by '.'
            entrypt = body.replace('/', '.');
        } else {
            String res = "";
            idx2 = 0;
            while (idx1!=-1) {
                res = body.substring(idx2, idx1+1) + res;
                idx2 = idx1+1;
                idx1 = body.indexOf('.', idx2);
            }

            int idx3 = body.indexOf('/', idx2);
            res = body.substring(idx2, idx3) + '.' + res;
            entrypt = res+body.substring(idx3+1).replace('/', '.');
        }

        entrypt = entrypt+"_IFact.create_factory";

        Object obj = TheClassLoader.newInstance(entrypt);
        if (obj==null) {
            // NOTE: log a message
            final String msg = "IGNORE (interceptor factory can not be created with entrypoint: "+entrypt+")";
            TheLogger.debug(_class_name, opname, msg);
            return null;
        } 

        // check if obj is effectively a interceptor factory
        if (!(obj instanceof CallInterceptorFactory)) {
            // NOTE: log a message
            final String msg = "IGNORE (object is not an interceptor factory: "+obj.getClass()+")";
            TheLogger.debug(_class_name, opname, msg);
            return null;
        }

        return (CallInterceptorFactory)obj;
    }

    //
    //  friend operations (for factory)
    //

    final protected void
    setHomePOA(org.omg.PortableServer.POA poa)
    {
        _home_poa = poa;
    }

    final protected void
    setComponentPOA(org.omg.PortableServer.POA poa)
    {
        _component_poa = poa;

        // obtain servant manager from POA
        try {
            _servant_activator = (ComponentServantActivatorImpl)poa.get_servant_manager();
        } catch (Exception ex) {
            // should not happen
            final String opname = "setComponentPOA";
            TheLogger.error(_class_name, opname, "FAILED", ex);
        }
    }

    //
    //  IDL:objectweb.org/corba/runtime/SystemComponent:1.0
    //

    final public void
    system_configuration_complete(SystemConfiguration cfg)
    {
        ComponentPOAConfiguration poacfg = (ComponentPOAConfiguration)cfg;

        // extract services
        _orb_service = poacfg.orb_service();
        _services_set = poacfg.services_set();

        // set it also to the servant activator
        _servant_activator.setConfiguration(_orb_service, _services_set);
    }

    final public void
    destroy()
    {
        // destroy the POA
        // here, we have set:
        //   - 'etherealize_objects' param as true
        //   - 'wait_for_completion' param as true
        _component_poa.destroy(true, true);
        _home_poa.destroy(true, true);
    }

    //
    //  IDL:objectweb.org/corba/runtime/SystemService:1.0
    //

    final public String
    service_id()
    {
        return _service_id;
    }

    //
    //  IDL:objectweb.org/ccm/runtime/ComponentPOAService:1.0
    //

    final public org.omg.Components.CCMHome
    activate_home(String uuid)
    {
        // create a dummy id
        byte[] id = Double.toString(Math.random()).getBytes();

        return activate_home_with_id(id, uuid);
    }

    final public org.omg.Components.CCMHome
    activate_home_with_id(byte[] id, String uuid)
    {
        final String opname = "activate_home_with_id";

        // load code archive
        // obtain archive location from component archive
        org.omg.CORBA.Object obj = null;
        org.coach.ECM.ExtComponentInstallation extci = null;
        org.coach.ECM.ComponentArchive carch = null;

        obj = _orb_service.resolve_initial_references("ExtComponentInstallation");
        extci = org.coach.ECM.ExtComponentInstallationHelper.narrow(obj);
        carch = extci.get_component_archive(uuid);

        // manage dependencies
        // valuefactories
        org.coach.ECM.ValuetypeFactoryDependency[] vfes = carch.get_valuetype_factories();
        org.coach.ECM.ValuetypeFactoryDependency vfe = null;
        for (int i=0;i<vfes.length;i++) {
            vfe = vfes[i];

            // add code location to class loader
            TheClassLoader.addResource(vfe.code_location);

            // create value factory
            Object ovf = TheClassLoader.newInstance(vfe.factory_entrypoint);
            org.omg.CORBA.portable.ValueFactory vf = (org.omg.CORBA.portable.ValueFactory)obj;

            // register
            ValueFactory fact = new ValueFactoryImpl(vfe.repid, vf);
            _orb_service.register_valuefactory(fact);    
        }


        // add code to class loader
        TheClassLoader.addResource(carch.code_location());

        // create instance from entrypoint
        // normally, it should be a HomeExecutor instance
        org.omg.Components.HomeExecutor hexe = null;
        Object jobj = TheClassLoader.newInstance(carch.code_entrypoint());

        // check if null
        if (jobj==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (invalid entrypoint)";
            TheLogger.error(_class_name, opname, msg);
        }

        if (jobj instanceof org.omg.Components.HomeExecutor) {
            hexe = (org.omg.Components.HomeExecutor)jobj;
        } else {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (not a Components::HomeExecutor)";
            TheLogger.error(_class_name, opname, msg);
        }

        // activate home
        // NOTE: must be done before any narrow (else OBJECT_NOT_EXISTS)
        // NOTE: the home executor MUST inherit from the ::Components::Servant
        if (hexe instanceof org.omg.Components.Servant) {
            try {
                org.omg.Components.Servant servant = (org.omg.Components.Servant)hexe;
                _home_poa.activate_object_with_id(id, servant.as_native_servant());
            } catch (Exception ex) {
                // NOTE: should not happen
                TheLogger.error(_class_name, opname, "FAILED", ex);
                return null;
            } 
        } else {
            TheLogger.error(_class_name, opname, "FAILED (not a CCM servant)");
            return null;
        }

        // create ref
        String hrepid = carch.home_repid();
        org.omg.CORBA.Object ref = _home_poa.create_reference_with_id(id, hrepid);
        org.omg.Components.CCMHome href = org.omg.Components.CCMHomeHelper.narrow(ref);

        // create ECM context
        // NOTE: the interceptor factory is given to the context (for stub interceptors creation)
        // and the servant activator (for sk interceptors creation)
        CallInterceptorFactory ifact = repid2Factory(hrepid);
        if (ifact!=null) {
            ifact.setContextInfo(_orb_service, _services_set, uuid);
        }

        HomeLocalEventsImpl levts = new HomeLocalEventsImpl(ifact);
        HomeLocalReceptaclesImpl lrecs = new HomeLocalReceptaclesImpl(ifact);
        // NOTE: TODO: how can we select the right context (monolithic or segmented ?)
        org.omg.Components.CCM2Context ctx = new HomeSession2ContextImpl(href, _orb_service, this, _services_set, levts, lrecs);

        // set context
        hexe.set_ccm_context(ctx);

        // register home to servant activator 
        _servant_activator.registerHome(id, uuid, hexe, ifact, ctx);

        return href;
    }

    final public void
    passivate_home(org.omg.Components.CCMHome href)
    {
        try {
            // deactivate so that future call leads to incarnation
            byte[] id = _home_poa.reference_to_id(href);
            _home_poa.deactivate_object(id);
        } catch (Exception ex) {
            // ignore (log debug message as this is nevertheless  not a normal behaviour)
            final String opname = "passivate_component";
            TheLogger.debug(_class_name, opname, "IGNORE", ex);
        }
    }

    //
    //  IDL:org/objectweb/ccm/runtime/ComponentPOAService:1.0
    //

    final public org.omg.CORBA.Object
    create_ref_with_id(byte[] id, String repid)
    {
        // register to servant activator
        _servant_activator.registerComponent(id);

        // create a ref
        return _component_poa.create_reference_with_id(id, repid);
    }

    final public byte[]
    get_id_from_ref(org.omg.CORBA.Object ref)
    throws InvalidReference
    {
        try {
            return _component_poa.reference_to_id(ref);
        } catch (Exception ex) {
            final String opname = "get_id_from_ref";
            TheLogger.debug(_class_name, opname, "FAILED (rethrown)", ex);
            throw new InvalidReference();
        } 
    }

    final public void
    passivate_component(byte[] id)
    {
        final String opname = "passivate_component";
        try {
            // deactivate so that future call leads to incarnation
            _component_poa.deactivate_object(id);
        } catch (Exception ex) {
            // ignore (log debug message as this is nevertheless  not a normal behaviour)
            TheLogger.debug(_class_name, opname, "IGNORE", ex);
        }
    }

    final public void
    remove_component(org.omg.Components.CCMObject comp)
    {
        final String opname = "remove_component";
        try {
            // get id from ref
            byte[] id = _component_poa.reference_to_id(comp);

            // unregister from servant activator
            _servant_activator.unregisterComponent(id);

            // deactivate so that future call leads to incarnation
            _component_poa.deactivate_object(id);
        } catch (Exception ex) {
            // ignore (log debug message as this is nevertheless  not a normal behaviour)
            TheLogger.debug(_class_name, opname, "IGNORE", ex);
        }
    }
}
