// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Default implementation of the <tt>ECA::ClientCallContext</tt> OMG IDL interface.</p>
 **/
public class ClientCallContextImpl
extends org.omg.CORBA.LocalObject
implements org.coach.ECA.ClientCallContext
{
    //
    static private String _class_name = "ClientCallContextImpl";
    private java.util.Hashtable _call_infos; // (thread id, ClientCallInformation)
    private ORBService _orb_service;
    private ServicesSet _services_set;
    private org.omg.CORBA.Object _real_stub;

    // default constructor
    public 
    ClientCallContextImpl(ORBService orbs, 
                          ServicesSet services,
                          org.omg.CORBA.Object rstub)
    {
        _orb_service = orbs;
        _services_set = services;
        _real_stub = rstub;
        _call_infos = new java.util.Hashtable();
    }

    //
    //  friend operations (for generated interceptor), keep public as in different packages
    //

    final public org.coach.ECA.ClientCallController
    popController()
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        return (org.coach.ECA.ClientCallController)callinfo.controller_stack.pop();
    }

    final public void
    pushController(org.coach.ECA.ClientCallController ctrl)
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        callinfo.controller_stack.push(ctrl);
    }

    final public boolean
    isEmpty()
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        return callinfo.controller_stack.isEmpty();
    }

    final public void
    addEntry(CallInformation cinfo, String opname)
    {
        // create storage
        ClientCallInformation callinfo = new ClientCallInformation();

        // fill
        callinfo.call_information = cinfo;
        callinfo.result = null;
        callinfo.system_exception = null;
        callinfo.user_exception = null;
        callinfo.eca_call_information.operation_name = opname;

        // store
        _call_infos.put(Thread.currentThread().getName(), callinfo);        
    }

    final public void
    removeEntry()
    {
        // remove from storage
        _call_infos.remove(Thread.currentThread().getName());
    }

    final public void
    setResult(org.omg.CORBA.Any res)
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        callinfo.result = res;
    }

    //
    //  IDL:coach.org/ECA/CallContext:1.0
    //

    final public org.coach.ECA.CallInformation
    call_information()
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        return callinfo.eca_call_information;
    }

    final public org.omg.CORBA.Any
    result()
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        return callinfo.result;
    }

    final public org.omg.CORBA.Any
    system_exception()
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        return callinfo.system_exception;
    }

    final public org.omg.CORBA.Any
    user_exception()
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        return callinfo.user_exception;
    }

    final public void
    send_system_exception(org.omg.CORBA.Any exc)
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        callinfo.system_exception = exc;
    }

    final public void
    send_user_exception(org.omg.CORBA.Any exc)
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        callinfo.user_exception = exc;
    }

    final public org.omg.CORBA.Object
    get_service_internal(String sid, String internal_id)
    {
        Service s = _services_set.find_service(sid);
        if (s==null) {
            return null;
        }

        return s.get_service_internal(internal_id);
    }

    final public org.omg.CORBA.Object
    get_service_callback(String sid, String callback_id)
    throws org.coach.ECM.UnknownService,
           org.coach.ECM.UnknownServiceCallback
    {
        // TODO: how can the reference to the segment which called
        // the operation be retrieved???

        // NOTE: one solution could be to have proxies specific to each segment and either wrapping the
        // references returned by LocalReceptacles::get_connections operation in the receptacle case
        // or wrapping the LocalReceptacles::push operation in the emitter/publisher cases
        // That way, the wrapper could "know" to which segment he's bound and set this information
        // in a thread-safe manner when called, then we could retrieved the information in this class
        // (as the thread executing the wrapper and the controllers is the same)

        throw new org.coach.ECM.UnknownService();
        /*
        Object obj = null;

        if (!(obj instanceof org.coach.ECM.ECMExecutor)) {
            throw new org.coach.ECM.UnknownService();
        }

        org.coach.ECM.ECMExecutor ecmexe = (org.coach.ECM.ECMExecutor)obj;
        return ecmexe.get_service_callback(sid, callback_id);
        */
    }

    final public org.coach.ECA.Slot
    get_slot(int slot_id)
    {
        return ComponentRuntime.getSlotFactory().getSlot(slot_id);
    }

    final public org.omg.CORBA.Any
    create_any()
    {
        return _orb_service.create_any();
    }

    //
    //  IDL:coach.org/runtime/ECA/ClientCallContext:1.0
    //

    final public org.omg.CORBA.Object
    target()
    {
        return _real_stub;
    }

    final public org.omg.CORBA.Policy
    get_request_policy(int ptype)
    {
        ClientCallInformation callinfo = (ClientCallInformation)_call_infos.get(Thread.currentThread().getName());
        CallInformation cinfo = callinfo.call_information;
        Policy[] pols = cinfo.policies;
        for (int i=0;i<pols.length;i++) {
            if (pols[i].corba_policy().policy_type()==ptype) {
                return pols[i].corba_policy();
            }
        }

        return null;
    }
}

class ClientCallInformation
{
    public org.omg.CORBA.Any result;
    public org.omg.CORBA.Any system_exception;
    public org.omg.CORBA.Any user_exception;
    public CallInformation call_information;
    public java.util.Stack controller_stack;
    public org.coach.ECA.CallInformation eca_call_information;

    public 
    ClientCallInformation()
    {
        result = null;
        system_exception = null;
        user_exception = null;
        call_information = null;
        controller_stack = new java.util.Stack();
        eca_call_information = new org.coach.ECA.CallInformation();
    }
}
