// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Default implementation of the <tt>ServicesSet</tt> interface.</p>
 **/
public class ServicesSetImpl
extends org.omg.CORBA.LocalObject
implements ServicesSet
{
    // services set
    static final private String _class_name = "ServicesSetImpl";
    private ServiceImpl[] _services;
    private CORBAServiceRef[] _degen_services;
    private ORBService _orb_service;

    // default constructor
    public
    ServicesSetImpl(ServiceImpl[] services,
                    CORBAServiceRef[] dservices,
                    ORBService orbs)
    {
        // services set
        _services = services;
        _degen_services = dservices;
        _orb_service = orbs;
    }

    //
    //  internal operations
    //

    private ServiceImpl
    findServiceById(String sid)
    {
        ServiceImpl[] services = _services;
        for (int i=0;i<services.length;i++) {
            if (services[i].service_id().equals(sid)) {
                return services[i];
            }
        }

        return null;
    }

    private PolicyImpl[]
    asPolicies(org.coach.ECM.CCDPolicyRef[] prefs)
    {
        // create policy object for each policy
        ServiceImpl service = null;
        PolicyImpl[] pols = new PolicyImpl[prefs.length];
        for (int i=0;i<prefs.length;i++) {
            // get service for policy
            service = findServiceById(prefs[i].service_id);

            // check if found
            if (service==null) {
                final String opname = "asPolicies";
                final String msg = "FAILED (service not found: "+prefs[i].service_id+")";
                TheLogger.error(_class_name, opname, msg);
                return null;
            }

            // create policy
            pols[i] = (PolicyImpl)service.create_policy(prefs[i].policyref, prefs[i].policyvalue);
        }

        return pols;
    }

    //
    //  IDL:objectweb.org/ccm/runtime/ServicesSet:1.0
    //

    final public Service
    find_service(String sid)
    {
        return findServiceById(sid);
    }

    final public org.omg.CORBA.Object
    find_degenerated_service(String regname)
    {
        // find ref on the ORB service
        org.omg.CORBA.Object ref = _orb_service.resolve_initial_references(regname);

        // check that the registration name correspond to a degenerated service
        CORBAServiceRef[] dservices = _degen_services;
        boolean isds = false;
        for (int i=0;i<dservices.length;i++) {
            if (dservices[i].orb_registration_name.equals(regname)) {
                isds = true;
                break;
            }
        }

        if (isds) {
            // NOTE: can ref be null is this case ? normally no
            return ref;
        }

        final String opname = "find_degenerated_service";
        final String msg = "regname is not a degenerated service: "+regname;
        TheLogger.debug(_class_name, opname, msg);

        return null;
    }

    final public Policy[]
    get_policies(String uuid, String sname)
    {
        // obtain component installation
        org.omg.CORBA.Object ref = null;
        org.coach.ECM.ExtComponentInstallation extci = null;

        ref = _orb_service.resolve_initial_references("ExtComponentInstallation");
        extci = org.coach.ECM.ExtComponentInstallationHelper.narrow(ref);

        // obtain component archive
        org.coach.ECM.ComponentArchive carch = extci.get_component_archive(uuid);

        // obtain policy refs
        org.coach.ECM.CCDPolicyRef[] prefs = carch.get_policies_for_target(sname);

        //
        return asPolicies(prefs);
    }
}