// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>Simple implementation of client and server request interceptors which log
 **  debug messages.</p>
 **/
public class DebugSRIImpl
extends org.omg.CORBA.LocalObject
implements org.omg.PortableInterceptor.ServerRequestInterceptor
{
    //
    static final private String _class_name = "DebugSRIImpl";

    //
    public 
    DebugSRIImpl()
    {
    }

    //
    //  IDL:omg.org/PortableInterceptor/RequestInterceptor:1.0
    //

    final public String
    name()
    {
        return _class_name;
    }

    final public void
    destroy()
    {
    }

    //
    //  IDL:omg.org/PortableInterceptor/ServerRequestInterceptor:1.0
    //

    final public void
    receive_request_service_contexts(org.omg.PortableInterceptor.ServerRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        String opname = "receive_request_service_contexts";
        TheLogger.debug(_class_name, opname, "");
        TheLogger.debug(_class_name, opname, "Request:   "+info.request_id());
        TheLogger.debug(_class_name, opname, "Operation: "+info.operation());
    }

    final public void
    receive_request(org.omg.PortableInterceptor.ServerRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        String opname = "receive_request";
        TheLogger.debug(_class_name, opname, "");
        TheLogger.debug(_class_name, opname, "Request:   "+info.request_id());
        TheLogger.debug(_class_name, opname, "Operation: "+info.operation());
        TheLogger.debug(_class_name, opname, "POA id:    "+info.adapter_id());
        TheLogger.debug(_class_name, opname, "Object id: "+info.object_id());
        TheLogger.debug(_class_name, opname, "Repid:     "+info.target_most_derived_interface());
    }

    final public void
    send_reply(org.omg.PortableInterceptor.ServerRequestInfo info)
    {
        String opname = "send_reply";
        TheLogger.debug(_class_name, opname, "");
        TheLogger.debug(_class_name, opname, "Request:   "+info.request_id());
        TheLogger.debug(_class_name, opname, "Operation: "+info.operation());
        TheLogger.debug(_class_name, opname, "POA id:    "+info.adapter_id());
        TheLogger.debug(_class_name, opname, "Object id: "+info.object_id());
    }

    final public void
    send_exception(org.omg.PortableInterceptor.ServerRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        String opname = "send_exception";
        TheLogger.debug(_class_name, opname, "");
        TheLogger.debug(_class_name, opname, "Request:   "+info.request_id());
        TheLogger.debug(_class_name, opname, "Operation: "+info.operation());
        // NOTE: normally, the exception should be NO_RESOURCES, but some ORBs send
        // BAD_INV_ORDER instead
        try {
            TheLogger.debug(_class_name, opname, "POA id:    "+info.adapter_id());
            TheLogger.debug(_class_name, opname, "Object id: "+info.object_id());
        } catch (org.omg.CORBA.BAD_INV_ORDER ex) {
            // NOTE: this can happen if the servant locator (if any) caused a location
            // forward or raised an exception
            TheLogger.debug(_class_name, opname, "BAD_INV_ORDER");
        } catch (org.omg.CORBA.NO_RESOURCES ex) {
            // NOTE: this can happen if the servant locator (if any) caused a location
            // forward or raised an exception
            TheLogger.debug(_class_name, opname, "NO_RESOURCES");
        }
    }

    final public void
    send_other(org.omg.PortableInterceptor.ServerRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        String opname = "send_other";
        TheLogger.debug(_class_name, opname, "");
        TheLogger.debug(_class_name, opname, "Request:   "+info.request_id());
        TheLogger.debug(_class_name, opname, "Operation: "+info.operation());
        // NOTE: normally, the exception should be NO_RESOURCES, but some ORBs send
        // BAD_INV_ORDER instead
        try {
            TheLogger.debug(_class_name, opname, "POA id:    "+info.adapter_id());
            TheLogger.debug(_class_name, opname, "Object id: "+info.object_id());
        } catch (org.omg.CORBA.BAD_INV_ORDER ex) {
            // NOTE: this can happen if the servant locator (if any) caused a location
            // forward or raised an exception
            TheLogger.debug(_class_name, opname, "BAD_INV_ORDER");
        } catch (org.omg.CORBA.NO_RESOURCES ex) {
            // NOTE: this can happen if the servant locator (if any) caused a location
            // forward or raised an exception
            TheLogger.debug(_class_name, opname, "NO_RESOURCES");
        }
    }
}
