// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>Default implementation of the <tt>FactoryFinderService</tt> system service.</p>
 **/
public class FactoryFinderServiceImpl
extends org.omg.CORBA.LocalObject
implements FactoryFinderService
{
    // factory finder
    static final private String _class_name = "FactoryFinderServiceImpl";
    static final private String _service_id = FactoryFinderService.SERVICE_ID;
    private java.util.Hashtable _factories;

    // default constructor
    public
    FactoryFinderServiceImpl()
    {
        // factory finder
        _factories = null;
    }

    //
    //  entry point
    //

    static public SystemService
    create_service()
    {
        return new FactoryFinderServiceImpl();
    }

    //
    //  IDL:objectweb.org/corba/runtime/SystemComponent:1.0
    //

    final public void
    system_configuration_complete(SystemConfiguration cfg)
    {
        _factories = new java.util.Hashtable();
    }

    final public void
    destroy()
    {
        // destroy all enclosed factories
        SystemComponent[] facts = (SystemComponent[])_factories.values().toArray(new SystemComponent[0]);
        for (int i=0;i<facts.length;i++) {
            facts[i].destroy();
        }

        // clear the table
        _factories.clear();
    }

    //
    //  IDL:objectweb.org/corba/runtime/SystemService:1.0
    //

    final public String
    service_id()
    {
        return _service_id;
    }

    //
    //  IDL:objectweb.org/corba/runtime/FactoryFinderService:1.0
    //

    final public void
    register_service_factory(String ssid, String location, String entrypt)
    {
        // check if location is set
        if (location!=null) { 
            // add archive location to class loader
            TheClassLoader.addResource(location);
        }

        // create the factory
        SystemFactory fact = (SystemFactory)TheClassLoader.newInstance(entrypt);

        // check if created
        if (fact==null) {
            // NOTE: TODO: should be reported as an exception
            final String opname = "register_service_factory";
            final String msg = "FAILED (factory not created)";
            TheLogger.error(_class_name, opname, msg);
        }

        // store
        _factories.put(ssid, fact);
    }

    final public SystemFactory
    find_service_factory(String ssid)
    {
        return (SystemFactory)_factories.get(ssid);
    }
}
