// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>Default implementation of the <tt>FileRegistrationScheme</tt> scheme.</p>
 **
 **  @see org.objectweb.corba.runtime.RegistrationServiceImpl RegistrationServiceImpl
 **/
public class FileRegistrationSchemeImpl
extends org.omg.CORBA.LocalObject
implements FileRegistrationScheme
{
    // scheme
    static final private String _class_name = "FileRegistrationSchemeImpl";
    static final private String _scheme_id = FileRegistrationScheme.SCHEME_ID;

    // default constructor
    protected
    FileRegistrationSchemeImpl()
    {
    }

    //
    //  entry point
    //

    static public RegistrationScheme
    create_scheme()
    {
        return new FileRegistrationSchemeImpl();
    }

    //
    //  IDL:objectweb.org/corba/runtime/RegistrationScheme:1.0
    //

    final public String
    scheme_id()
    {
        return _scheme_id;
    }

    //
    //  IDL:objectweb.org/corba/runtime/FileRegistrationScheme:1.0
    //

    final public void
    write_ior(String name, org.omg.CORBA.Object ref, ORBService orbs)
    {
        // use name as iorfile
        String iorfile = name;

        // stringify IOR
        String sref = orbs.object_to_string(ref);

        // write reference to the file
        java.io.FileOutputStream out = null;
        try {
            out = new java.io.FileOutputStream(iorfile);
            out.write(sref.getBytes());
            out.close();
        } catch(Exception ex) {
            // ignore
            final String opname = "write_ior";
            final String msg = "IGNORE (file: "+iorfile+")";
            TheLogger.debug(_class_name, opname, msg, ex);
        }
    }

    final public org.omg.CORBA.Object
    read_ior(String name, ORBService orbs)
    {
        // use name as iorfile
        String iorfile = name;

        // read reference from the file
        java.io.BufferedReader in = null;
        StringBuffer res = new StringBuffer();
        try {
            in = new java.io.BufferedReader(new java.io.FileReader(iorfile));
            while(in.ready()) {
                res.append(in.readLine());
            }

            in.close();
        } catch(Exception ex) {
            final String opname = "read_ior";
            final String msg = "FAILED (file: "+iorfile+")";
            TheLogger.error(_class_name, opname, msg, ex);
        }

        // unstringify IOR
        return orbs.string_to_object(res.toString());
    }
}
