// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>Default implementation of the <tt>INSRegistrationScheme</tt> scheme.</p>
 **
 **  @see org.objectweb.corba.runtime.RegistrationServiceImpl RegistrationServiceImpl
 **/
public class INSRegistrationSchemeImpl
extends org.omg.CORBA.LocalObject
implements INSRegistrationScheme
{
    // scheme
    static final private String _class_name = "INSRegistrationSchemeImpl";
    static final private String _scheme_id = INSRegistrationScheme.SCHEME_ID;

    // default constructor
    protected
    INSRegistrationSchemeImpl()
    {
    }

    //
    //  entry point
    //

    static public RegistrationScheme
    create_scheme()
    {
        return new INSRegistrationSchemeImpl();
    }

    //
    //  IDL:objectweb.org/corba/runtime/RegistrationScheme:1.0
    //

    final public String
    scheme_id()
    {
        return _scheme_id;
    }

    //
    //  IDL:objectweb.org/corba/runtime/INSRegistrationScheme:1.0
    //

    final public void
    bind(String name, org.omg.CORBA.Object ref, ORBService orbs)
    {
        // obtain NS
        org.omg.CORBA.Object obj = orbs.resolve_initial_references("NameService");

        // check if NS is available
        if (obj==null) {
            final String opname = "bind";
            final String msg = "IGNORE (no NS)";
            TheLogger.debug(_class_name, opname, msg);
            return ;
        }

        bind_in_ns(name, ref, obj);
    }

    final public void
    bind_in_ns(String name, org.omg.CORBA.Object ref, org.omg.CORBA.Object obj)
    {
        final String opname = "bind_in_ns";

        org.omg.CosNaming.NamingContext ns = null;
        try {
            ns = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        } catch (org.omg.CORBA.BAD_PARAM ex) {
            final String msg = "IGNORE (not a NS)";
            TheLogger.debug(_class_name, opname, msg, ex);
            return ;
        }

        // use name as INS name
        String insname = name;

        // bind
        org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
        ncomp[0] = new org.omg.CosNaming.NameComponent(insname, "");
        try {
            ns.rebind(ncomp, ref);
        } catch (Exception ex) {
            final String msg = "IGNORE (INS name: "+insname+")";
            TheLogger.debug(_class_name, opname, msg, ex);
            return ;
        }
    }

    final public void
    unbind(String name, ORBService orbs)
    {
        // obtain NS
        org.omg.CORBA.Object obj = orbs.resolve_initial_references("NameService");

        // check if NS is available
        if (obj==null) {
            final String opname = "unbind";
            final String msg = "IGNORE (no NS)";
            TheLogger.debug(_class_name, opname, msg);
            return ;
        }

        unbind_in_ns(name, obj);
    }

    final public void
    unbind_in_ns(String name, org.omg.CORBA.Object obj)
    {
        final String opname = "unbind_in_ns";

        org.omg.CosNaming.NamingContext ns = null;
        try {
            ns = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        } catch (org.omg.CORBA.BAD_PARAM ex) {
            // ignore
            final String msg = "IGNORE (not a NS)";
            TheLogger.debug(_class_name, opname, msg, ex);
            return ;
        }

        // use name as INS name
        String insname = name;

        // unbind
        org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
        ncomp[0] = new org.omg.CosNaming.NameComponent(insname, "");
        try {
            ns.unbind(ncomp);
        } catch (Exception ex) {
            // ignore
            final String msg = "IGNORE (INS name: "+insname+")";
            TheLogger.debug(_class_name, opname, msg, ex);
            return ;
        }
    }

    public org.omg.CORBA.Object
    lookup(String name, ORBService orbs)
    {
        // obtain NS
        org.omg.CORBA.Object obj = orbs.resolve_initial_references("NameService");

        // check if NS is available
        if (obj==null) {
            // ignore
            final String opname = "lookup";
            final String msg = "FAILED (no NS)";
            TheLogger.error(_class_name, opname, msg);
            return null;
        }

        return lookup_in_ns(name, obj);
    }

    public org.omg.CORBA.Object
    lookup_in_ns(String name, org.omg.CORBA.Object obj)
    {
        final String opname = "lookup_in_ns";

        org.omg.CosNaming.NamingContext ns = null;
        try {
            ns = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        } catch (org.omg.CORBA.BAD_PARAM ex) {
            final String msg = "FAILED (not a NS)";
            TheLogger.error(_class_name, opname, msg, ex);
            return null;
        }

        // use name as INS name
        String insname = name;

        // lookup
        org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
        ncomp[0] = new org.omg.CosNaming.NameComponent(insname, "");
        org.omg.CORBA.Object res = null;
        try {
            res = ns.resolve(ncomp);
        } catch (Exception ex) {
            final String msg = "FAILED (INS name: "+insname+")";
            TheLogger.error(_class_name, opname, msg, ex);
            return null;
        }

        return res;
    }
}
