// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.logservice;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Implementation of a client request interceptor for the log service.
 **  This CRI propagates the 'id' information of the current to the server side.</p>
 **
 **  @see org.objectweb.corba.logservice.LogCurrentImpl LogCurrentImpl
 **  @see org.objectweb.corba.logservice.LogSRIImpl LogSRIImpl
 **/
public class LogCRIImpl
extends org.omg.CORBA.LocalObject
implements org.omg.PortableInterceptor.ClientRequestInterceptor
{
    //
    private org.omg.PortableInterceptor.ORBInitInfo _info;
    private org.omg.IOP.Codec _codec;
    private int _slot_id;

    public
    LogCRIImpl(org.omg.PortableInterceptor.ORBInitInfo info,
               org.omg.IOP.Codec codec,
               int slotid)
    {
        _info = info;
        _codec = codec;
        _slot_id = slotid;
    }

    //
    //  internal operations
    //

    private org.omg.PortableInterceptor.Current
    getPICurrent()
    {
        org.omg.CORBA.Object obj = null;
        org.omg.PortableInterceptor.Current current = null;
        try {
            obj = _info.resolve_initial_references("PICurrent");
        } catch (org.omg.PortableInterceptor.ORBInitInfoPackage.InvalidName ex) {
        } catch (Exception ex) {
            System.err.println("!!! LogCRIImpl::getPICurrent exception caught: "+ex.getMessage());
        } catch (Error err) {
            System.err.println("!!! LogCRIImpl::getPICurrent error caught: "+err.getMessage());
        }

        return org.omg.PortableInterceptor.CurrentHelper.narrow(obj);
    }

    private void
    setIdInServiceContext(org.omg.PortableInterceptor.ClientRequestInfo info)
    {
        // get client id from slot
        org.omg.CORBA.Any cid = null;
        try {
            cid = info.get_slot(_slot_id);
        } catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            System.err.println("!!! LogCRIImpl::setIdInServiceContext: invalid slot");
        }

        if ((cid==null) || (cid.type().kind()==org.omg.CORBA.TCKind.tk_null)) {
            System.err.println("!!! LogCRIImpl::setIdInServiceContext: slot has no value");
            return ;
        }

        // set client id in service context
        org.omg.IOP.ServiceContext sctx = new org.omg.IOP.ServiceContext();
        sctx.context_id = LOG_IOP_SERVICE_ID.value;
        try {
            sctx.context_data = _codec.encode(cid);
        } catch (org.omg.IOP.CodecPackage.InvalidTypeForEncoding ex) {
            System.err.println("!!! LogCRIImpl::setIdInServiceContext: invalid type for encoding in service context");
        }

        info.add_request_service_context(sctx, true);
    }

    private void
    getIdFromServiceContext(org.omg.PortableInterceptor.ClientRequestInfo info)
    {
        // get log service context
        org.omg.IOP.ServiceContext sctx = null;
        try {
            sctx = info.get_reply_service_context(LOG_IOP_SERVICE_ID.value);
        } catch (org.omg.CORBA.SystemException ex) {
            System.err.println("!!! LogCRIImpl::getIdFromServiceContext: Service context not found");
        }

        if (sctx==null) {
            System.err.println("!!! LogCRIImpl::getIdFromServiceContext: Service context not found");
            return ;
        }

        // extract server id from context
        org.omg.CORBA.Any cid = null;
        try {
            cid = _codec.decode(sctx.context_data);
        } catch (org.omg.IOP.CodecPackage.FormatMismatch ex) {
        }

        // set server id in slot
        try {
            getPICurrent().set_slot(_slot_id, cid);
        } catch (org.omg.PortableInterceptor.InvalidSlot ex) {
        }
    }

    //
    //  IDL:omg.org/PortableInterceptor/RequestInterceptor:1.0
    //

    final public String
    name()
    {
        return "LogCRIImpl";
    }

    final public void
    destroy()
    {
    }

    //
    //  IDL:omg.org/PortableInterceptor/ClientRequestInterceptor:1.0
    //

    final public void
    send_request(org.omg.PortableInterceptor.ClientRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        System.err.println("### LogCRIImpl::send_request called on operation: "+info.operation());
        // insert the client id in the request
        setIdInServiceContext(info);
    }

    final public void
    send_poll(org.omg.PortableInterceptor.ClientRequestInfo info)
    {
        System.err.println("### LogCRIImpl::send_poll called on operation: "+info.operation());
        // insert the client id in the request
        setIdInServiceContext(info);
    }

    final public void
    receive_reply(org.omg.PortableInterceptor.ClientRequestInfo info)
    {
        System.err.println("### LogCRIImpl::receive_reply called on operation: "+info.operation());
        // extract the server id from the reply
        getIdFromServiceContext(info);
    }

    final public void
    receive_exception(org.omg.PortableInterceptor.ClientRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        System.err.println("### LogCRIImpl::receive_exception called on operation: "+info.operation());
        // extract the server id from the reply
        getIdFromServiceContext(info);
    }

    final public void
    receive_other(org.omg.PortableInterceptor.ClientRequestInfo info)
    throws org.omg.PortableInterceptor.ForwardRequest
    {
        System.err.println("### LogCRIImpl::receive_other called on operation: "+info.operation());
        // extract the server id from the reply
        getIdFromServiceContext(info);
    }
}
